/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.adom;

/*
 * ADOMInsertService.java
 *
 * Created on February 10, 2003, 1:04 PM
 */

import java.util.Iterator;
import java.util.Properties;
import java.util.Enumeration;
import org.moremotion.action.ActionServiceContext;

import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.exception.MoreMotionException;
import org.moremotion.action.ActionService;

import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ADOMInsertService.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ADOMInsertService implements ActionService {

  public void doService(ActionServiceContext asc)  throws ServiceException, java.io.IOException {

    MoreMotionRequest request = asc.getRequest();
    
    String resbundle = "mor_adom";
    
    try {

      String adomname = request.getParameter("adom");
      if (adomname == null || adomname.equals("")) {
        throw new MoreMotionException("ADOM_PARAMETER_MISSING",null,null, resbundle);
      }
      
      String adomscope = request.getParameter("scope","2");

      String nextpage = request.getParameter("nextpage");
      if (nextpage == null || nextpage.equals("")) {
        throw new MoreMotionException("NEXTPAGE_PARAMETER_MISSING",null,null, resbundle);
      }
      String keyfield = request.getParameter("keyfield","");
      String incrfield = request.getParameter("incrfield","");

      ADOM adom = null;
      if (adomscope.charAt(0) == '1') adom = request.getRequestADOM(adomname);
      else if (adomscope.charAt(0) == '2') adom = request.getSessionADOM(adomname);
      else if (adomscope.charAt(0) == '3') adom = request.getApplicationADOM(adomname);

      if (adom == null) {
        if (adomscope.charAt(0) == '1') adom = request.newRequestADOM(adomname);
        else if (adomscope.charAt(0) == '2') adom = request.newSessionADOM(adomname);
        else if (adomscope.charAt(0) == '3') adom = request.newApplicationADOM(adomname);
        //        adom.setNodeValue("count","0");
        adom.setNodeValue("incr","0");
      }

      ADOMNode itemnode = null;
      int incrfieldval = 0;
      if (!incrfield.equals("")) {
        String ifval = request.getParameter("f_" + incrfield,"");
        if (ifval.equals("")) {
          throw new MoreMotionException("INCR_FIELD_MISSING",new String[] {incrfield},null, resbundle);
        }
      }
      if (!keyfield.equals("")) {
        String kfval = request.getParameter("f_" + keyfield,"");
        if (kfval.equals("")) {
          throw new MoreMotionException("KEY_FIELD_MISSING",new String[] {keyfield},null, resbundle);
        }
        if (asc.logger.LEVEL1) asc.logger.print("kfval : " + kfval);

        try {
          Iterator it = adom.getNodes();
          while (it.hasNext()) {
            ADOMNode node = (ADOMNode)it.next();
            if (node.getName().equals("item")) {
              if (asc.logger.LEVEL1) asc.logger.print("keyfield value : " + node.getNodeValue(keyfield));
              if (node.getNodeValue(keyfield).equals(kfval)) {
                itemnode = node;
                try {
                  incrfieldval = Integer.parseInt(node.getNodeValue(incrfield));
                } catch (Exception e) {};
                break;
              }
            }
          }
        } catch(Exception e) {}
      }

      if (itemnode == null) {
        itemnode = new ADOMNode("item");
        adom.addNode(itemnode);
        int nodeid = 0;
        try {
          nodeid = Integer.parseInt(adom.getNodeValue("incr"));
          nodeid++;
        } catch (Exception e) {};
        adom.setNodeValue("incr","" + nodeid);
        itemnode.addAttribute("id","" + nodeid);
      }

      Properties fields = new Properties();

      Enumeration en = request.getParameterNames();
      while (en.hasMoreElements()) {
        String fn = (String)en.nextElement();
        if (fn.startsWith("f_")) {
          String fv = request.getParameter(fn);
          fn = fn.substring(2);
          if (incrfieldval != 0 && incrfield.equals(fn)) {
            try {
              fv = "" + (Integer.parseInt(fv) + incrfieldval);
            } catch (Exception ex) {}
          }
          if (asc.logger.LEVEL1) asc.logger.print("fn: " + fn + ", fv: " + fv );
          fields.put(fn,fv);
        }
      }

      itemnode.setNodeValues(fields);
      request.generatePage(nextpage);

    } catch (Exception e) {
      if (asc.logger.LEVEL1) e.printStackTrace();
      request.createMessage(e);
      request.generateDefaultErrorPage();
      return;
    }

  }

}
