/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * MultiPartAjaxResponse.java
 *
 * Created on 09 September 2007 Sunday, 03:26
 *
 */

package org.moremotion.mmcomp.ajax;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import javax.servlet.http.HttpServletResponse;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.ConfigurationException;
import org.moremotion.datasrc.DataSourceConfig;
import org.moremotion.datasrc.DataSourceManager;
import org.moremotion.datasrc.Paging;
import org.moremotion.exception.ExceptionUtils;
import org.moremotion.exception.MoreMotionException;
import org.moremotion.resource.ResourceException;
import org.moremotion.resource.ResourceManager;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.StringParser;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: MultiPartAjaxResponse.java 108 2008-03-25 11:03:29Z erkan $
 */
public class MultiPartAjaxResponse {
  
  private MoreMotionRequest request;
  private PrintWriter out;
  private ArrayList responses;
  
  /**
   * Creates a new instance of MultiPartAjaxResponse
   */
  public MultiPartAjaxResponse(MoreMotionRequest request) throws IOException {
    
    this.request = request;
    responses = new ArrayList();
    
    HttpServletResponse response = request.getResponse();
    String enc = request.getParameter("_resp_enc","iso-8859-1");
    response.setContentType("text/html; charset="+ enc);
    out = request.getResponse().getWriter();
  }
  
  public void addResponse(AjaxResponse response) {
    responses.add(response);
  }
  
  public void build() throws ConfigurationException {
    addResultFields();
    addPagingInfo();
    for (int i = 0; i < responses.size(); i++) {
      AjaxResponse response = (AjaxResponse)responses.get(i);
      if (i > 0) {
        out.write("$_RESPONSE_SEPARATOR_$");
      }
      out.write(response.getResponseString());
    }
  }
  
  public void flush() {
    out.close();
  }
  
  /**
   * Add an AjaxResponse that contains the Result Fields to this object.
   * The Result fields are set with the Ajax MScript function "setResultField(name,value)". If during the execution of the
   * current request, "setResultFiel()" function is used then this function will return a JSON structure enclosed with
   * <CODE>&lt;&lt;RESULT-FIELDS:</CODE> and <CODE>:RESULT-FIELDS&gt;&gt;</CODE> delimiters in order to let javascript object "AjaxRequest" to
   * create a "ResultFields" object out of it and pass back to the callers.
   */
  private void addResultFields() {
    
    HashMap resultFields = (HashMap)request.getAttribute("ajax.result.fields");
    
    if (resultFields == null) {
      return;
    }
    
    AjaxResponse response = new AjaxResponse();
    response.setName("*ResultFields");
    response.setType("*ResultFields");
    
    Iterator itr = resultFields.entrySet().iterator();
    while (itr.hasNext()) {
      Map.Entry entry = (Map.Entry)itr.next();
      String name = (String) entry.getKey();
      String value = (String) entry.getValue();
      response.addReponseProperty(name,escape(value));
    }
    responses.add(response);
    
  }
  
  private void addPagingInfo() throws ConfigurationException {
    
    String[] defs = request.getParameterValues("_refreshds");
    if (defs == null) return;
    
    StringBuffer pinfo = new StringBuffer();
    
    AjaxResponse response = new AjaxResponse();
    response.setName("*PagingInfo");
    response.setType("*PagingInfo");
    
    for (int i = 0; i < defs.length; i++) {
      String def = defs[i];
      StringParser par = new StringParser(def,';');
      while (par.hasMore()) {
        String s = par.nextItem();
        if (s.startsWith("ds:")) {
          String dsname = XString.after(s,':',null);
          if (dsname != null) {
            DataSourceConfig dsconfig = DataSourceManager.getDataSourceConfig(request, dsname);            
            Paging paging = new Paging(request,dsname,dsconfig);
            if (paging != null) {
              response.addReponseProperty(dsname, paging.getPagingInfo());
            }
          }
          break;
        }
      }
    }
    responses.add(response);
    
  }
  
  public void buildError(Exception exception) {
    
    try {
      java.sql.SQLException se = null;
      ADOMNode msgNode;
      String unitName;
      if (exception instanceof MoreMotionException) {
        MoreMotionException e = (MoreMotionException)exception;
        se = e.getOriginalSQLException();
        unitName = e.getResBundleName();
        msgNode = createMessage(unitName, e.getMessage(), e.getArguments(), e);
      } else {
        msgNode = createMessage(null, exception.getMessage(), null, exception);
        if (exception instanceof java.sql.SQLException) {
          se = (java.sql.SQLException)exception;
        }
      }
      
      if (se != null) {
        msgNode.setNodeValue("jdbc-error-code",se.getErrorCode() + "");
        msgNode.setNodeValue("jdbc-sql-state",se.getSQLState());
      }
      
      String errorText = generateMessageText(msgNode, request.getLanguage());
      out.write("!ERROR:" + errorText);
      
    } catch (Exception e) {
      e.printStackTrace();
    }
    
  }
  
  public void buildErrorFromMesageAdoms() {
    
    String adomNames = (String)request.getAttribute("moremotion.message.adom.names");
    if (adomNames == null) {
      return;
    }
    
    StringBuffer buf = new StringBuffer();
    StringParser parser = new StringParser(adomNames,',');
    while (parser.hasMore()) {
      String adomName = parser.nextItem();
      ADOM adom = request.getRequestADOM(adomName);
      Iterator itr = adom.getRootNode().getNodes();
      while (itr.hasNext()) {
        ADOMNode msgNode = (ADOMNode)itr.next();
        try {
          String text = generateMessageText(msgNode, request.getLanguage());
          if (buf.length() > 0) {
            buf.append("\n\n");
          }
          buf.append(text);
        } catch (Exception e) {
          e.printStackTrace();
        }
      }
      
    }
    
    out.write("!ERROR:" + buf.toString());
    
  }
  
  
  private String generateMessageText(ADOMNode n, String language) throws ResourceException {
    
    StringBuffer buf = new StringBuffer();
    // buf.append("MOREMOTION AJAX SERVICE ERROR!\n\n"); 
    
    String text = n.getNodeValue("text");
    text = ResourceManager.replaceResDefinitions(text,language);
    buf.append( text + "\n");
    
    String jerrc = n.getNodeValue("jdbc-error-code");
    if (jerrc.length() < 0) buf.append("JDBC Error Code : " + jerrc + "\n");
    
    String jsqls = n.getNodeValue("jdbc-sql-state");
    if (jsqls.length() < 0) buf.append("JDBC SQL State  : " + jsqls + "\n");
    
    String det = n.getNodeValue("details");
    if (det.length() > 0) {
      buf.append("\n" +
          ResourceManager.replaceResDefinitions(det,language)
          );
    }
    
    
    String details = buf.toString();
    details = XString.beforeLast(details,"org.moremotion.servlet.JspFilter");
    
    int x = details.lastIndexOf("org.moremotion");
    if (x > -1) {
      x = details.indexOf("\r\n",x+10);
      if (x > 0) details = details.substring(0,x);
    }
    
    return details;
    
  }
  
  private ADOMNode createMessage(String unitName, String messageId, String[] arguments, Object details)
  throws Exception {
    
    
    Exception exception = null;
    String detailsStr = null;
    
    if (details instanceof Exception) {
      exception = (Exception)details;
    } else if (details instanceof String) {
      detailsStr = (String)details;
    }
    
    if (messageId == null || messageId.length() == 0) {
      messageId = "Error!";
    }
    
    String resbundName = "moremotion";
    
    if (unitName != null) {
      resbundName = XString.before(unitName,'.');
    }
    
    String adomname = resbundName + "_messages";
    ADOM adom = request.getRequestADOM(adomname);
    if (adom == null) {
      adom = request.newRequestADOM(adomname);
      adom.setResBundleName(resbundName);
    }
    ADOMNode messageNode = new ADOMNode("message");
    adom.addNode(messageNode);
    
    messageNode.addAttribute("id",messageId);
    
    String messageText = "";
    if (messageId != null && messageId.length() > 0) {
      messageText = (new org.moremotion.resource.Resource(messageId,arguments,resbundName)).format(request.getLanguage());
    }
    messageNode.setNodeValue("unitname",unitName);
    messageNode.setNodeValue("text",messageText);
    if (exception != null) {
      messageNode.setNodeValue("details", ExceptionUtils.getStackTrace(exception));
    } else if (details != null) {
      messageNode.setNodeValue("details", detailsStr);
    }
    
    return messageNode;
  }
  
  
  public static String escape(String s) {
    
    StringBuffer b = null;
    
    int ln = s.length();
    
    for (int i = 0; i < ln; i++) {
      
      char c = s.charAt(i);
      char nc = i+1 < ln ? s.charAt(i+1) : ' ';
      
      switch (c) {
        
        case '\'':
          if (b == null) {
            b = new StringBuffer(s.length() + 10);
            b.append(s.substring(0,i));
          }
          b.append("\\'");
          break;
          
        case '\r':
          if (b == null) {
            b = new StringBuffer(s.length() + 10);
            b.append(s.substring(0,i));
          }
          if (nc == '\n') break;
          b.append("\\n");
          break;
          
        case '\n':
          if (b == null) {
            b = new StringBuffer(s.length() + 10);
            b.append(s.substring(0,i));
          }
          b.append("\\n");
          break;
          
        default:
          if (b != null) {
            b.append(c);
          }
          
      }
    }
    
    if (b == null) return s;
    return b.toString();
    
  }
  
}
