/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * RefreshBlocksActionService.java
 *
 * Created on 09 September 2007 Sunday, 01:47
 *
 */

package org.moremotion.mmcomp.ajax;

import java.io.IOException;
import org.moremotion.action.ActionService;
import org.moremotion.action.ActionServiceContext;
import org.moremotion.adom.ADOM;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigManager;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.ServiceException;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: RefreshBlocksActionService.java 199 2008-09-08 08:48:37Z erkan $
 */
public class RefreshBlocksActionService implements ActionService {
  
  public void doService(ActionServiceContext asc) throws ServiceException, IOException {
    
    MoreMotionRequest request = asc.getRequest();
    MultiPartAjaxResponse mpar = new MultiPartAjaxResponse(request);
    
    try {
      
      String cfgname = request.getParameter("_cfg");
      Config config = ConfigManager.getConfig("actionParams", cfgname, request.getOriginPage() );
      if (config == null) {
        throw new AjaxException("AJAX_ACTION_MISSING",new String[]{cfgname},null);
      }
      
      String dsname = config.getParameter("dataSources").resolve(request).stringValue();
      String nte = config.getParameter("nodesToEncrypt").resolve(request).stringValue(null); 
      String responseHandler = config.getParameter("responseHandler").resolve(request).stringValue("AjaxMgr.refreshBlocksResponseHandler");
      
      String[] dsnames = XString.split(dsname,',');
      String[] ntes = XString.split(nte == null ? "" : nte,';');
      
      for (int i = 0; i < dsnames.length; i++) {
        dsname = dsnames[i];
        nte = i < ntes.length - 1 ? ntes[i] : null;
        
        ADOM adom = Utils.generateADOM(dsname,request,nte);
        
        AjaxResponse response = new AjaxResponse();
        response.setADOM(adom);
        response.addReponseProperty("blockSeq",i+"");
        response.setResponseHandler(responseHandler);
        
        mpar.addResponse(response);
        
      }
      mpar.build();
      
    } catch (Exception ex) {
      mpar.buildError(ex);
      
    } finally {
      mpar.flush();
      
    }
    
  }
  
}
