/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.filer.upload;

import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.Locale;
import java.util.StringTokenizer;
import javax.servlet.ServletInputStream;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigManager;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * @author Mehmet Tastan
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Uploader.java 51 2008-03-10 11:46:54Z erkan $
 */

public class Uploader {
  
  private static final boolean debug = true;

  final static int NORMAL = 0;
  final static int AMPERSAND = 1;
  final static int AMPERSHARP = 2;
  private final static int BUFFER_SIZE = 2048;

  private UploadManager um = null;
  private ServletInputStream sis = null;
  private HashMap map = null;
  private long sizeLimit = 100000000000L;
  private MoreMotionRequest req = null;

  private String checkpoint = null;
  private String location = null;
  private String namingMethod = "?";
  boolean overwrite = true;
  private String serverFileName = "";
  private String fileboxId = null;

  public Uploader(UploadManager um) {
    this.um = um;
  }

  public String getServerFileName() {
    return this.serverFileName;
  }

  public void upload(MoreMotionRequest req) throws Exception {

    this.req = req;

    long totalSize = 0L;

    String fileLocation = null;
    int contentLength = req.getContentLength();

    // RFC 1867
    String contentType = req.getContentType();
    //System.out.println("contentType " + contentType);
    //PrintWriter out = res.getWriter();

    if (contentType == null) {
      um.Message("UPLOADER:CONTENT_TYPE_NULL");
      return;
    }

    String BOUNDARY_LABEL = "boundary=";
    int ind = contentType.toLowerCase().indexOf(BOUNDARY_LABEL);
    if (ind == -1) {
      um.Message("UPLOADER:BOUNDARY_LABEL_MISSING", BOUNDARY_LABEL);
      //      um.Message("No '" + BOUNDARY_LABEL + "' found in the content type");
      return;
    }

    String boundary = contentType.substring(ind + BOUNDARY_LABEL.length());
    //    if (boundary.trim().length() == 0) {
    //      um.Message("UPLOADER:BOUNDARY_EMTY");
    //      return;
    //    }

    String boundaryString = "--" + boundary;

    sis = req.getInputStream();

    byte[] buffer = new byte[BUFFER_SIZE];
    map = new HashMap();
    int result = sis.readLine(buffer, 0, BUFFER_SIZE);

    outer:
      while (true) {

        if (result <= 0) {
          um.Message("UPLOADER:STREAM_TRUNCATED");
          break;
        }

        String line = new String(buffer, 0, result);
        if (!line.startsWith(boundaryString)) {
          um.Message("UPLOADER:MULTIPART_BOUNDARY_MISSING");
          break;
        }

        if (line.substring(boundaryString.length()).startsWith("--")) {
          break;
        }

        result = sis.readLine(buffer, 0, BUFFER_SIZE);
        if (result <= 0) {
          um.Message("UPLOADER:BOUNDARY_HAS_NO_CONTENT");
          break;
        }

        line = new String(buffer, 0, result);

        StringTokenizer tokenizer = new StringTokenizer(line, ";\r\n");
        String token = tokenizer.nextToken();

        Locale currentLocale = Locale.US;
        String lowerToken = token.toLowerCase(currentLocale);

        if (!lowerToken.startsWith("content-disposition")) {
          um.Message("UPLOADER:CONTENT_DISPOSITION_MISSING");
          //          System.out.println("Format error. 'Content-Disposition' expected."+ lowerToken);
          break;
        }

        String disposition = lowerToken.substring(21);
        if (!disposition.equals("form-data")) {
          um.Message("UPLOADER:UNKNOWN_DISPOSITION",disposition);
          //          if (debug) System.out.println("Sorry, I don't know how to handle [" + disposition + "] disposition.");
          break;
        }

        if (tokenizer.hasMoreElements()) {
          token = tokenizer.nextToken();
        } else {
          um.Message("UPLOADER:NAME_EXPECTED");
          break;
        }

        int nameStart = token.indexOf("name=\"");
        int nameEnd = token.indexOf("\"", nameStart+7);
        if (nameStart < 0 || nameEnd < 0) {
          um.Message("UPLOADER:NAME_EXPECTED");
          break;
        }
        String name = token.substring(nameStart+6, nameEnd);
        //System.out.println("name = " + name);

        if (tokenizer.hasMoreElements()) {
          String filename = null;
          int fnStart, fnEnd;
          File file = null;
          String fileContentType = null;
          int size = 0;

          fnStart = line.indexOf("filename=\"");

          if (fnStart < 0) { // filename term missing

            um.Message("UPLOADER:FILE_NAME_DEFINITION_MISSING");
            result = sis.readLine(buffer, 0, BUFFER_SIZE);
            continue;
          }
          fnEnd = line.indexOf("\"", fnStart + 11);
          if (fnEnd < 0) {

            um.Message("UPLOADER:FILE_NAME_MISSING");
            break;
          }
          else {
            filename = line.substring(fnStart+10, fnEnd);
            int lastindex = -1;
            if ((lastindex = filename.lastIndexOf('/')) < 0) {
              lastindex = filename.lastIndexOf('\\');
            }
            if (lastindex >= 0) {
              filename = filename.substring(lastindex+1);
            }
            filename = processEscape(filename);
          }

          result = sis.readLine(buffer, 0, BUFFER_SIZE);
          if (result <= 0) {
            um.Message("UPLOADER:STREAM_TRUNCATED");
            break;
          }

          fileContentType = new String(buffer, 0, result);
          if (fileContentType.toUpperCase().startsWith("CONTENT-TYPE:")) {
            fileContentType = fileContentType.substring(13).trim();
          } else {
            um.Message("UPLOADER:FILE_CONTENT_TYPE_MISSING");

          }

          try {
            byte[] tmpbuffer1 = buffer;
            byte[] tmpbuffer2 = new byte[BUFFER_SIZE];
            byte[] tmpbuffer = tmpbuffer2;
            int tmpbufferlen = 0;
            boolean isFirst = true;
            boolean odd = true;
            inner:
              while ((result = sis.readLine(buffer, 0, BUFFER_SIZE)) > 0) {

                if (isFirst) { // ignore all proceeding \r\n
                  if (result == 2 && buffer[0] == '\r' && buffer[1] == '\n') {
                    continue;
                  }
                  //                  bout = new ByteArrayOutputStream();

                  if (!createServerFile()) {
                    return;
                    //                    continue outer;
                  }
                }

                if (bytesStartsWith(buffer, 0, result, boundaryString)) {
                  if (!isFirst) {
                    size += tmpbufferlen - 2;
                    totalSize += tmpbufferlen - 2;

                    if (totalSize > sizeLimit) throw new SizeLimitExceededException("");

                    um.writeFile(tmpbuffer, tmpbufferlen - 2);
                    //                    if (bout != null) {
                    //                      bout.write(tmpbuffer, 0, tmpbufferlen - 2);
                    //                    }
                  }
                  um.closeFile(serverFileName);
                  continue outer;
                } else {
                  if (!isFirst) {
                    size += tmpbufferlen;
                    totalSize += tmpbufferlen - 2;

                    if (totalSize > sizeLimit) throw new SizeLimitExceededException("");
                    um.writeFile(tmpbuffer, tmpbufferlen);
                    //                    if (bout != null) {
                    //                      bout.write(tmpbuffer, 0, tmpbufferlen);
                    //                    }
                  }
                }
                if (odd) {
                  buffer = tmpbuffer2;
                  tmpbuffer = tmpbuffer1;
                } else {
                  buffer = tmpbuffer1;
                  tmpbuffer = tmpbuffer2;
                }
                odd = !odd;
                tmpbufferlen = result;
                isFirst = false;
              }
              um.closeFile(serverFileName);
          } catch (SizeLimitExceededException sle) {
            um.eraseFile();
            um.Message("UPLOADER:SIZE_LIMIT_EXCEEDED", (sizeLimit / 1024) + "KB");
            sle.printStackTrace();
            return;
          } catch (IOException ie) {
            um.Message("UPLOADER:IO_ERROR", ie.getMessage());
            ie.printStackTrace();
          } finally {
            if (size > 0) {
              appendValue(name, filename, fileContentType, size);
            }
          }

          result = sis.readLine(buffer, 0, BUFFER_SIZE);
          if (debug) {
            //            System.out.println("What should I read here? - result = " + result + ", read [" + new String(buffer, 0, result) + "]");
          }
        } else { // no more elements
          result = sis.readLine(buffer, 0, BUFFER_SIZE);
          if (result <= 0) {
            um.Message("UPLOADER:STREAM_TRUNCATED");
            break;
          }
          result = sis.readLine(buffer, 0, BUFFER_SIZE);
          if (result <= 0) {
            um.Message("UPLOADER:STREAM_TRUNCATED");
            break;
          }
          String value = new String(buffer, 0, result-2); // exclude \r\n
          appendValue(name, value);
        }

        result = sis.readLine(buffer, 0, BUFFER_SIZE);
      }

      return;
  }

  private String processEscape(String string) {

    StringBuffer buffer = new StringBuffer(string.length());
    char[] chars = string.toCharArray();
    StringBuffer escaped = new StringBuffer(6);
    int status = NORMAL;

    for (int i = 0; i < string.length(); i++) {
      switch (status) {
        case NORMAL :
          if (chars[i] == '&') {
            status = AMPERSAND;
          } else {
            buffer.append(chars[i]);
          }
          break;

        case AMPERSAND :
          if (chars[i] == '#') {
            status = AMPERSHARP;
          } else {
            status = NORMAL;
            buffer.append('&');
          }
          break;

        case AMPERSHARP :
          if (chars[i] == ';') {
            try {
              buffer.append((char) Integer.parseInt(escaped.toString()));
            } catch (NumberFormatException nfe) {
              buffer.append(escaped);
              buffer.append(';');
            }
            escaped.setLength(0);
            status = NORMAL;
          } else {
            escaped.append(chars[i]);
          }
          break;
      }
    }

    if (escaped.length() > 0) {
      buffer.append(escaped);
    }

    return buffer.toString();
  }

  boolean bytesStartsWith(byte[] bytes, int offset, int length, String toCompare) {
    boolean result = true;
    if (toCompare.length() > length) {
      return false;
    }

    for (int i = toCompare.length() - 1; i >= 0; i--) {
      if (toCompare.charAt(i) != bytes[offset + i]) {
        result = false;
        break;
      }
    }
    return result;
  }

  void appendValue(String name, String value, String contentType, int size) {
    UploadData data = new UploadData(name, value, contentType, size, true);
    map.put(name, data);
  }

  void appendValue(String name, String value) {
    UploadData data = new UploadData(name, value, null, 0, false);
    map.put(name, data);
  }


  private String getOneHourLaterId() {

    SimpleDateFormat format = new SimpleDateFormat("yyyyMMddHHmmss");

    Date d = new Date();

    GregorianCalendar cal = new GregorianCalendar();
    cal.setTimeInMillis(d.getTime());
    cal.add(Calendar.HOUR,1);

    d.setTime( cal.getTimeInMillis() );

    return format.format(d);

  }

  public String getFileBoxId() {
    return this.fileboxId;
  }

  private boolean createServerFile() throws Exception {

    UploadData ud = null;

    ud = (UploadData) map.get("fbi");
    this.fileboxId = ud.value;

    ud = (UploadData) map.get("uli");

    getParameters(ud.value);

    if (org.moremotion.security.SecurityManager.checkAccessAuthority(req,checkpoint,null) == false) {
      um.Message("UPLOADER:INSUFFICIENT_AUTHORITY");
      return false;
    }

    ud = (UploadData) map.get("lfn");
    String lfn = ud.value;

    lfn = lfn.replace('\\', '/');

    int li = lfn.lastIndexOf('/');
    String fn = lfn.substring(li+1);

    li = fn.lastIndexOf('.');
    String ext = fn.substring(li+1);
    String pfn = fn.substring(0,li);

    char nm = (namingMethod + "1").charAt(0);

    switch (nm) {
      case '1': break;
      case '2': fn = getOneHourLaterId() + "." + ext; break;
      case '3': fn = getOneHourLaterId() + "_" + fn; break;
      default : fn = pfn + "_" + getOneHourLaterId() + "." + ext; break;
    }

    this.serverFileName = fn;

    um.setWorkingPath(location);

    try {
      if (!um.openFile(fn, overwrite)) { return false; }
    } catch (Exception e) {
      um.Message("UPLOADER:UNABLE_TO_CREATE_FILE", fn, location, e.getMessage());
      return false;
    }
    return true;

  }

  public String getCurrentPageName() {
    UploadData ud = (UploadData) map.get("cpn");
    return (ud != null) ? ud.value : "index";
  }


  private void getParameters(String uploadId) throws Exception {

    Config config = ConfigManager.getConfig("fileUpload",uploadId);
    
    if (config == null) return;
    
    checkpoint = config.getParameter("checkPoint").resolve(req).stringValue(null);
    location = config.getParameter("location").resolve(req).stringValue("");
    namingMethod = config.getParameter("namingMethod").resolve(req).stringValue("");
    String szl = config.getParameter("sizeLimit").resolve(req).stringValue("*");
    overwrite = config.getParameter("overwrite").resolve(req).booleanValue(true);

    try {
      sizeLimit = Long.parseLong(szl) * 1024;
    } catch (Exception e) {}

  }


  public static void  main(String[] args) throws Exception {

    String x = "abc";
    int ln = x.indexOf("abc");

    String y = x.substring(ln + x.length());
    System.out.println(">" + y + "<");

  }


}

