/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * MailTask.java
 *
 * Created on 31 September 2005 Monday , 11:58
 */

package org.moremotion.mmcomp.mailer;


import java.util.LinkedList;
import java.util.Properties;
import javax.mail.Authenticator;
import javax.mail.PasswordAuthentication;
import javax.mail.Session;
import javax.mail.internet.InternetAddress;
import org.moremotion.util.Logger;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: MailTask.java 68 2008-03-17 07:25:27Z erkan $
 */
public class MailTask {
  
  private String smtpHost = null;
  private String smtpUser = null;
  private String smtpFrom = null;
  private String smtpPassword = null;
  private int smtpPort = 25;
  boolean smtpAuth = false;
  boolean smtpSecure = false;
  private LinkedList mel = null;
  private int entryIndex = -1;
  private int cnt = 0;
  private boolean entriesComplete = false;
  private boolean htmlMail = false;
  private Session session = null;
  private String charset = "iso-8859-1";
  private int pauseCount = -1;
  private int pauseDuration = -1;
  private Logger logger = null;
  private MailLogger mlogger = null;
  private int sentQty = 0;
  
  public MailTask(String host, int port, boolean authRequired, boolean secure, String user, String password, String from) throws MailerException {
    mel = new LinkedList();
    smtpHost = host;
    smtpPort = port;
    smtpAuth = authRequired;
    smtpSecure = secure;
    smtpUser = user;
    smtpPassword = password;
    smtpFrom = from;
    if (smtpSecure) {
      java.security.Security.addProvider( new com.sun.net.ssl.internal.ssl.Provider());
      java.security.Security.setProperty( "ssl.SocketFactory.provider", "org.moremotion.security.domain.DummySSLSocketFactory");
    }
    session = getSession();
  }
  
  public void setMailLogger(MailLogger mlogger) {
    this.mlogger = mlogger;
  }
  
  public MailLogger getMailLogger() {
    return this.mlogger;
  }
  
  public void setLogger(Logger logger) {
    this.logger = logger;
  }
  
  public Logger getLogger() {
    return this.logger;
  }
  
  
  public synchronized int getSendQty() {
    return ++sentQty;
  }
  
  public void addEntry(int seqnr, String from, String to, String cc, String bcc, String subject, String message, Attachment[] attachs, Object object) {
    MailEntry me = new MailEntry();
    me.seqnr = seqnr;
    me.from = from;
    me.to = to;
    me.cc = cc;
    me.bcc = bcc;
    me.subject = subject;
    me.message = message;
    me.attachs = attachs;
    me.object = object;
    synchronized (this) {
      mel.addLast(me);
    }
  }
  
  public void clear() {
    synchronized (this) {
      mel.clear();
    }
  }
  
  public void setHtmlMail(boolean value) {
    this.htmlMail = value;
  }
  
  public boolean getHtmlMail() {
    return this.htmlMail;
  }
  
  public void setCharset(String charset) {
    this.charset = charset;
  }
  
  public String getCharset() {
    return this.charset;
  }
  
  public void setEntriesComplete() {
    synchronized (this) {
      this.entriesComplete = true;
    }
  }
  
  public boolean getEntriesComplete() {
    return this.entriesComplete;
  }
  
  public MailEntry getNextEntryToProcess() {
    
    synchronized (this) {
      
      if (pauseCount != -1 && cnt > pauseCount) {
        cnt = 0;
        boolean timePassed = false;
        long t1 = System.currentTimeMillis();
        long t3 = pauseDuration;
        t3 = t3 * 1000L;
        while (!timePassed) {
          long t2 = System.currentTimeMillis();
          if ((t2 - t1) > t3) {
            timePassed = true;
          }
        }
      }
       
      boolean result = this.entriesComplete;
      if ((entryIndex + 1) < mel.size()) {
        entryIndex++;
        cnt++;
        return (MailEntry)mel.get(entryIndex);
      }
      return null;
    }
  }
  
  
  public String setPauseSettings(String str) {
    
    int i = str.indexOf("pause(");
    if (i == -1) return str;
    
    String retval = str.substring(0,i);
    
    String s = new String(str.substring(i + 6));
    i = s.indexOf(')');
    if (i == -1) return retval;
    s = s.substring(0,i);
    
    try {
      pauseCount = Integer.parseInt(XString.before(s,',',null));
      pauseDuration = Integer.parseInt(XString.after(s,',',null));
    } catch(Exception e) {}
    
    return retval;
  }
  
  public Session getSession() throws MailerException {
    
    try {
      new InternetAddress(smtpUser);
      
      Session result = null;
      
      Properties props = new Properties();
      props.put("mail.transport.protocol", "smtp");
      props.put("mail.smtp.user", smtpUser);
      props.put("mail.smtp.host", smtpHost);
      if (smtpFrom == null || smtpFrom.length() == 0) {
        smtpFrom = smtpUser;
      }
      
      if (smtpSecure) {
        String SSL_FACTORY = "org.moremotion.security.domain.DummySSLSocketFactory";
        props.setProperty( "mail.smtp.socketFactory.class", SSL_FACTORY);
        props.setProperty( "mail.smtp.socketFactory.fallback", "false");
        props.setProperty( "mail.smtp.socketFactory.port", "" + smtpPort);
      }
      
      props.put("mail.from", smtpFrom);
      props.put("mail.smtp.auth", String.valueOf(smtpAuth));
      //    props.put("mail.debug", "true");
      
      if (smtpAuth) {
        result = Session.getInstance(props, new SimpleAuthenticator(smtpUser, smtpPassword));
      } else {
        result = Session.getInstance(props);
      }
      return result;
      
    } catch (Exception ex) {
      throw new MailerException(ex);
    }
    
  }
  
  
  static class SimpleAuthenticator extends Authenticator {
    String username, password;
    
    /** Simple authenticator for connecting SMTP server.
     * @param username User name.
     * @param password Password.
     */
    public SimpleAuthenticator(String username, String password) {
      this.username = username;
      this.password = password;
    }
    /** Returns PasswordAuthentication object to connect to SMPT server.
     * @return PasswordAuthentication object.
     */
    protected PasswordAuthentication getPasswordAuthentication() {
      return new PasswordAuthentication(username, password);
    }
  }
  
}
