/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * TextMailSender.java
 *
 * Created on 31 September 2005 Monday, 19:23
 */

package org.moremotion.mmcomp.mailer;

import javax.mail.MessagingException;
import java.io.File;
import java.util.Date;
import javax.activation.DataHandler;
import javax.activation.DataSource;
import javax.activation.FileDataSource;
import javax.mail.Message;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;
import javax.mail.internet.MimeUtility;

import org.moremotion.util.Logger;
import org.moremotion.util.XString;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: TextMailSender.java 87 2008-03-21 19:49:11Z erkan $
 */
public class TextMailSender {
  
  public static void send(Session ses, Transport trans, MailTask mtask, MailEntry me, String caller) throws Exception {
    
    String charset = mtask.getCharset();
    Logger logger = mtask.getLogger(); 
    MailLogger mlogger = mtask.getMailLogger();
    
    Message msg = new MimeMessage(ses);
    
    MailAddress to = new MailAddress(me.to);
    MailAddress cc = new MailAddress(me.cc);
    MailAddress bcc = new MailAddress(me.bcc);
    
    msg.setSubject(MimeUtility.encodeText( me.subject, charset, "B" ));
    msg.setSentDate(new Date());
    
    if (me.attachs == null || me.attachs.length == 0) {
      msg.setContent(me.message, "text/plain;charset=\"" + charset + "\"");
      
    } else {
      
      Multipart multipart = new MimeMultipart();
      
      //fill message
      MimeBodyPart messageBodyPart = new MimeBodyPart();
      messageBodyPart.setText(me.message, charset);
      multipart.addBodyPart(messageBodyPart);
      
      // Add attachements
      for (int i = 0; i < me.attachs.length; i++) {
        
        String an = me.attachs[i].name;
        File f = me.attachs[i].file;
        
        DataSource source = new FileDataSource(f);
        DataHandler handler = new DataHandler(source);
        if (an.length() == 0) an = f.getName();
        
        messageBodyPart = new MimeBodyPart();
        messageBodyPart.setDataHandler( handler );
        messageBodyPart.setFileName(an);
        multipart.addBodyPart(messageBodyPart);
      }
      msg.setContent(multipart);
    }
    try {
      
      if (me.from.length() > 0) {
        String person = XString.before(me.from,'<',"").trim(); 
        String fromAddr = XString.after(me.from,'<');
        fromAddr = XString.before(fromAddr,'>');
        if (person.length() > 0) {
          msg.setFrom(new InternetAddress(fromAddr,person,charset));
        } else {
          msg.setFrom(new InternetAddress(fromAddr));
        }
      }
      
      // msg.setFrom(new InternetAddress( MimeUtility.encodeText( me.from, charset, "B" )) );
      
      if (to.hasValidAddress()) {
        msg.setRecipients(Message.RecipientType.TO, to.getAddresses());
      }
      
      if (cc.hasValidAddress()) {
        msg.setRecipients(Message.RecipientType.CC, cc.getAddresses());
      }
      if (bcc.hasValidAddress()) {
        msg.setRecipients(Message.RecipientType.BCC, bcc.getAddresses());
      }

      if (to.hasValidAddress()) {
        trans.sendMessage(msg, to.getAddresses());
      }
        
      Utils.report(mtask, to, mlogger, logger);
      
      if (cc.hasValidAddress()) {
        trans.sendMessage(msg, cc.getAddresses());
      }
      Utils.report(mtask, cc, mlogger, logger);
      
      if (bcc.hasValidAddress()) {
        trans.sendMessage(msg, bcc.getAddresses());
      }
      Utils.report(mtask, bcc, mlogger, logger);
      
    } catch (MessagingException ex) {
      if (mlogger != null) mlogger.print("Invalid address " + ex.getMessage());
      if (logger.LEVEL1) logger.print("Invalid address " + ex.getMessage() );
      return;
    }
    
  }
  
}
