/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * BuildFilterProcess.java
 *
 * Created on 21 May 2006 Sunday, 11:12
 */

package org.moremotion.mmcomp.reldb;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.evaluator.EvalUtils;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.evaluator.SymbolResolvingException;
import org.moremotion.process.ProcessException;
import org.moremotion.process.Process;
import org.moremotion.process.ProcessRecord;
import org.moremotion.util.StringParser;
import org.moremotion.util.StringParserEx;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: BuildFilterProcess.java 215 2008-09-13 10:27:24Z erkan $
 */
public class BuildFilterProcess extends Process {
  
  private ConfigParameter fields;
  private ConfigParameter exclFields;
  private ConfigParameter operator;
  private ConfigParameter targetSymbol;
  private ConfigParameter upperFunc;
  private int count = 0;
  private String expr = "";
  private String TargetSymbol = "";
  
  
  public void prepare() throws ProcessException {
    
    if (logger.LEVEL1) {
      logger.printres("*PROCESS_STARTED", callDef.getStepName());
    }
    
    try {
      fields = processConfig.getParameter("fields");
      exclFields = processConfig.getParameter("exclfields");
      operator = processConfig.getParameter("opr");
      targetSymbol = processConfig.getParameter("tarsymbol");
      upperFunc = processConfig.getParameter("upperfunc");
      
      TargetSymbol = targetSymbol.resolve(request).stringValue();
      
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }
    
    if (logger.LEVEL1) {
      logger.printres("*PROCESS_PARAMETERS_LINE");
      logger.printres("BUILD_FILTER_PROCESS_PARAMETERS",
          new String[] { fields+"", exclFields+"", operator+"", targetSymbol+"", upperFunc+""} );
    }
    
    
  }
  
  public void processRecord(ProcessRecord prec) throws ProcessException {
    
    String Fields;
    String ExclFields;
    String Operator;
    String UpperFunc;
    
    try {
      Fields = fields.resolve(request).stringValue("*");
      ExclFields = exclFields.resolve(request).stringValue("");
      Operator = operator.resolve(request).stringValue();
      UpperFunc = upperFunc.resolve(request).stringValue(null);
      if (UpperFunc != null && UpperFunc.length() > 0 && UpperFunc.indexOf("(") == -1) {
        UpperFunc = UpperFunc + "(@)";
      }
      
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }
    
    if (logger.LEVEL2) {
      if (fields.hasMScript() || exclFields.hasMScript() || operator.hasMScript()) {
        logger.printres("*PROCESS_PARAMETERS_LINE_RESOLVED");
        logger.printres("BUILD_FILTER_PROCESS_PARAMETERS",
            new String[]{Fields, ExclFields, Operator, UpperFunc} );
      }
    }
    
    HashMap fnames = new HashMap();
    
    StringParser sp = new StringParser(Fields,',');
    sp.trimmed();
    while (sp.hasMore()) {
      String f = sp.nextItem();
      String dt = XString.before(f, ':', " ");
      f = XString.after(f, ':', f);
      if (f.equals("*")) {
        String[] names = prec.getBlock().getFieldNames();
        for (int i = 0; i < names.length; i++) {
          String fx = names[i];
          fnames.put(fx,dt + fx);
        }
      } else {
        String sqlName = XString.after(f,'/');
        f = XString.before(f,'/');
        fnames.put(f,dt + pureName(sqlName));
      }
    }
    
    sp = new StringParser(ExclFields,',');
    sp.trimmed();
    while (sp.hasMore()) {
      String f = sp.nextItem();
      fnames.remove(f);
    }
    
    StringBuffer filter = new StringBuffer(200);
    
    Iterator itr = fnames.entrySet().iterator();
    
    while (itr.hasNext()) {
      
      Map.Entry entry = (Map.Entry)itr.next();
      String fname = (String)entry.getKey();
      String s = (String)entry.getValue();
      String sqlName = s.substring(1);
      
      char dt = s.charAt(0);
      String value = prec.getField(pureName(fname), "");
      
      if (value.length() > 0) {
        
        if (filter.length() > 0) {
          filter.append(" " + Operator.toLowerCase() + " ");
        }
        String likeL = "";
        String likeR = "";
        
        if (fname.length() > 1 && fname.startsWith("%")) {
          likeL = "%";
          fname = fname.substring(1);
        }
        if (fname.length() > 1 && fname.endsWith("%")) {
          likeR = "%";
          fname = fname.substring(0,fname.length()-1);
        }
        filter.append( parse(value, sqlName, likeL, likeR, dt, UpperFunc) );
      }
    }
    
    if (filter.length() > 0) {
      count++;
      if (expr.length() > 0) {
        expr = expr + " or ";
      }
      expr = expr + "(" + filter.toString() + ")";
    }
    
    
  }
  
  public void finalizeProcess() throws ProcessException {
    
    if (getCompletionCode() >= 8) return;
    
    if (logger.LEVEL2) logger.printres("*PROCESS_FINALIZING",callDef.getStepCaption());
    
    if (count > 1) {
      expr = "(" + expr + ")";
    }
    
    MMSymbolResolver mmsr = request.getSymbolResolver();
    
    try {
      mmsr.setValue(TargetSymbol, expr);
    } catch (SymbolResolvingException ex) {
      throw new ProcessException(ex);
    }
    
  }
  
  
  private static final String[] operators = {"like ", "LIKE ", "<=", ">=", "<", ">","="};
  
  
  private static String pureName(String str) {
    if (str.startsWith("%")) str = str.substring(1);
    if (str.endsWith("%")) str = str.substring(0,str.length()-1);
    return str;
  }
  
  
  private static String parse(String expr, String name, String likeL, String likeR, char dt, String upperFunc) {
    
    StringBuffer buf = new StringBuffer();
    
    StringParserEx pbc = new StringParserEx(expr, new String[]{"or","OR","and","AND"},new char[]{' ','('}, new char[]{' ',')'});
    pbc.trimmed();
    int count = 0;
    
    while (pbc.hasMore()) {
      count++;
      
      String t = pbc.nextItem();
      
      if (t.startsWith("(") && t.endsWith(")")) {
        buf.append( " (" + parse(t.substring(1,t.length()-1), name, likeL, likeR, dt, upperFunc) + ")");
      } else {
        
        String opr = "=";
        for (int i = 0; i < operators.length; i++) {
          
          if (t.startsWith(operators[i])) {
            opr = operators[i].toLowerCase();
            t = t.substring( operators[i].length() ).trim();
            break;
          }
        }
        
        if ((likeL + likeR).length() > 0) {
          opr = "like";
        }
        if ( !t.startsWith("'") && !t.endsWith("'") && t.indexOf('%') > -1) {
          opr = "like";
          likeL = "";
          likeR = "";
        }
        if (buf.length() > 0) buf.append(' ');
        String fn = name;
        String fv = getValue(t, likeL, likeR, dt);
        if (dt == 'u' && upperFunc != null) {
          buf.append( getUpper(upperFunc,fn) + " " + opr + " " + getUpper(upperFunc,fv) );
          
        } else {
          buf.append( fn + " " + opr + " " + fv);
        }
        
      }
      
      String orand = pbc.getDelimiter().toLowerCase();
      if (orand.length() > 0 && buf.length() > 0) {
        buf.append(" " + orand);
      }
    }
    if (count > 1) {
      return "(" + buf.toString() + ")";
    } else {
      return buf.toString();
    }
  }
  
  private static String getUpper(String upperFunc, String parm) {
    return XString.replace(upperFunc,"@",parm);
  }
  
  private static String getValue(String s, String likeL, String likeR, char dt) {
    
    if (dt == ' ') {
      if (EvalUtils.isNumber(s)) {
        return s;
      } else if (EvalUtils.isBoolean(s)) {
        return s;
      } else {
        if (s.startsWith("'") && s.endsWith("'")) {
          return "'" + likeL + EvalUtils.doubleTheQuotes(s.substring(1,s.length()-1)) + likeR + "'";
        }
        return "'" + likeL + EvalUtils.doubleTheQuotes(s) + likeR + "'";
      }
    } else {
      if (dt == 's' || dt == 'u') {
        if (s.startsWith("'") && s.endsWith("'")) {
          return "'" + likeL + EvalUtils.doubleTheQuotes(s.substring(1,s.length()-1)) + likeR + "'";
        }
        return "'" + likeL + EvalUtils.doubleTheQuotes(s) + likeR + "'";
      } else {
        return s;
      }
    }
    
  }
  
}
