/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.reldb;

/*
 * ReldbUpdateProcess.java
 *
 * Created on February 10, 2003, 1:04 PM
 */

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigManager;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigurationException;
import org.moremotion.exception.ExceptionUtils;
import org.moremotion.process.ProcessException;
import org.moremotion.process.Process;
import org.moremotion.process.ProcessRecord;
import org.moremotion.evaluator.MScriptResolver;
import org.moremotion.util.JdbcConnectionException;
import org.moremotion.util.JdbcConnection;
import org.moremotion.util.JdbcUtils;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ReldbUpdateProcess.java 215 2008-09-13 10:27:24Z erkan $
 */
public class ReldbUpdateProcess extends Process {
  
  private ConfigParameter conName;
  private ConfigParameter query;
  
  private Connection con = null;
  private Statement stmt = null;
  private CallableStatement cstmt = null;
  
  private boolean doublebs;
  private String ignoreChars;
  private boolean ignoreCR;
  private boolean ignoreLF;
  
  private String Query;
  private String querypart = null;
  
  
  public void prepare() throws ProcessException {
    
    if (logger.LEVEL1) {
      logger.printres("*PROCESS_STARTED", callDef.getStepName());
    }
    
    try {
      conName = processConfig.getParameter("conn");
      query = processConfig.getParameter("query");
      
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }
    
    if (logger.LEVEL2) {
      logger.printres("*PROCESS_PARAMETERS_LINE");
      logger.printres("UPDATE_PROCESS_PARAMETERS", conName.toString(), query.toString() );
    }
    
    try {
      
      String ConName = conName.resolve(request).stringValue();
      
      /*
       * Get Connection Configuration
       */
      Config conConfig;
      try {
        conConfig = ConfigManager.getConfig("jdbcConnection", ConName);
        if (conConfig == null) {
          throw new ProcessException(this,"CONNECTION_CONFIG_MISSING",new String[] {ConName},null);
        }
      } catch (ConfigurationException ex) {
        throw new ProcessException(this,"CONNECTION_CONFIG_INVALID",new String[] {ConName},ex);
      }
      
      /*
       * Try to connect
       */
      try {
        con = JdbcConnection.getConnection(request, conConfig);
      } catch (JdbcConnectionException ex) {
        throw new ProcessException(this,"UNABLE_TO_CONNECT_WITH_CON",new String[] {ConName},ex);
      }
      if (logger.LEVEL1) {
        logger.printres("JDBC_CONNECTION_ESTABLISHED",new String[]{ConName});
      }
      
      // Double Backslash support
      String Driver = conConfig.getParameter("driver").resolve(request).stringValue();
      doublebs = Driver.indexOf("mysql") > -1 ||
          conConfig.getParameter("doubleBS").resolve(request).booleanValue(false);
      
      ignoreChars =
          conConfig.getParameter("ignoreChars").resolve(request).stringValue(null);
      
      ignoreCR = ignoreChars != null && ignoreChars.equalsIgnoreCase("cr");
      ignoreLF = ignoreChars != null && ignoreChars.equalsIgnoreCase("lf");

      /*
       * Get the SQL Query without resolving.
       * It will be resolved statement by statement.
       */
      Query = query.stringValue();
    
    } catch (ConfigParameterException ex) {
      throw new ProcessException(ex);
    }
    
  }
  
  
  public void processRecord(ProcessRecord prec) throws ProcessException {
    
    try {
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESSING_RECORD", ""+prec.getIndex() );
      }
      
      MScriptResolver resolver = new MScriptResolver(Query, request.getSymbolResolver(), ';');
      resolver.setSqlInjectionCheck(true);
      
      if (doublebs) {
        resolver.setDoubleBackSlashes(true);
      }
      
      querypart = resolver.nextStatement();
      
      while ( querypart != null) {
        
        if (ignoreCR) {
          querypart = querypart.replaceAll("\r","");
        } else if (ignoreLF) {
          querypart = querypart.replaceAll("\n","");
        }
        
        if (logger.LEVEL2) {
          logger.printres("QUERY_PART", querypart);
        }
        
        if (querypart.startsWith("selecT")) {
          /*
           * Special Query
           */
          querypart = JdbcUtils.arrangeSpecialSQL(querypart);
          querypart = querypart.replaceAll("vpool:","vpool_");
          
          if (stmt == null) {
            stmt = con.createStatement();
            if (logger.LEVEL2) {
              logger.printres("STATEMENT_PREPARED_FOR_selecT_QUERY");
            }
          }
          
          ResultSet specialRs = stmt.executeQuery(querypart);
          
          if (specialRs != null) {
            JdbcUtils.setFields(specialRs, prec, request, logger);
          }
          
        } else if ( querypart.startsWith("{") && querypart.endsWith("}") ) {
          
            /*
             * A Stored Procedure Call
             */
          if (logger.LEVEL2) {
            logger.printres("STORED_PROCEDURE_CALL_REQUESTED");
          }
          
          if (cstmt != null) cstmt.close();
          cstmt = con.prepareCall(querypart);
          if (logger.LEVEL2) {
            logger.printres("CALLABLE_STATEMENT_PREPARED");
          }
          cstmt.execute();
          
        } else {
          
          if (stmt == null) {
            stmt = con.createStatement();
            if (logger.LEVEL2) {
              logger.printres("STATEMENT_PREPARED");
            }
          }
          stmt.execute(querypart);
          if (logger.LEVEL2) logger.printres("QUERY_EXECUTED_SUCCESSFULLY");
          
        }
        
        querypart = resolver.nextStatement();
      }
      
      resolver.setSqlInjectionCheck(false);
      
      
    } catch (Exception ex) {
      
      try {
        if (stmt != null) stmt.close();
        if (cstmt != null) cstmt.close();
      } catch (SQLException sqle) {
        sqle.printStackTrace();
      }
      
      if (logger.LEVEL1) {
        logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      }

      logger.printres("SQL_EXECUTION_ERROR_IN_PROCESS",callDef.getStepCaption(), ExceptionUtils.getStackTrace(ex),querypart); 
      
      throw new ProcessException(ex);
    }
  }
  
  public void finalizeProcess() throws ProcessException {
    
    if (logger.LEVEL2) {
      logger.printres("*PROCESS_FINALIZING",callDef.getStepName());
    }
    
    try {
      if (stmt != null) {
        stmt.close();
        stmt = null;
      }
      
      if (cstmt != null) {
        cstmt.close();
        cstmt = null;
      }
      
    } catch (SQLException ex) {
      throw new ProcessException(ex);
    }
    
  }
  
  
}
