/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.security;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Properties;
import org.moremotion.datasrc.DataService;
import org.moremotion.datasrc.DataSourceConfig;

import org.moremotion.datasrc.Paging;
import org.moremotion.datasrc.DataServiceContext;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.Config;
import org.moremotion.security.SecurityContext;
import org.moremotion.security.User;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.XString;
import org.moremotion.security.domain.SignedUser;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ActiveUserList.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ActiveUserList implements DataService {

  public void doService(ADOM adom, DataServiceContext dsc) throws ServiceException {

    try {

      adom.clear();

      MoreMotionRequest request = dsc.getRequest();
      DataSourceConfig dsconfig = dsc.getDataSourceConfig();
      String secdom = Utils.getSecurityDomainName(request, dsconfig);
      Config sdconfig = Utils.getSecurityDomainConfig(secdom);
      SecurityContext sctx = Utils.getSecurityContext(request, sdconfig);

      Paging paging = dsc.getPaging();

      boolean fullDetails = dsconfig.getParameter("FullDetails").resolve(request).booleanValue(false);
      String sdname = dsc.getDataSourceConfig().getParameter("Subdomain").resolve(request).stringValue("");

      int auc = sctx.getActiveUserCount(sdname);
      adom.setNodeValue("count", "" + auc);

      ArrayList users = null;

      if (paging.getItemsPerPage() > 0) {
        paging.setNumberOfItems(auc);
        users = sctx.getActiveUsers(sdname, paging.getSkipAmount() + 1, paging.getItemsPerPage());
      } else {
        users = sctx.getActiveUsers(sdname, 1, 999999);
      }

      for (int i = 0; i < users.size(); i++) {
        SignedUser su = (SignedUser) users.get(i);

        String username = su.userName;
        ADOMNode item = new ADOMNode("item");
        adom.addNode(item);
        String sdn = XString.before(username, '/', null);
        username = XString.after(username, '/');
        item.setNodeValue("username", username);
        item.setNodeValue("sdname", sdn);
        item.setNodeValue("signIn", su.getSignInTime() + "");
        item.setNodeValue("lastAccess", su.getLastReferredTime() + "");

        if (sdn != null) {
          item.setNodeValue("subdomain", sdn);
        }
        if (fullDetails) {
          addFullDetails(sdn, username, item, sctx);
        }
      }
    } catch (Exception e) {
      throw new ServiceException(e);
    }
  }

  private void addFullDetails(String subdom, String username, ADOMNode usernode, SecurityContext sctx) throws Exception {

    if (subdom != null) {
      sctx.setVar("subdomain", subdom);
    }
    User user = sctx.getUser(username);

    usernode.setNodeValue("fullname", user.getFullname());
    usernode.setNodeValue("email", user.getEmail());
    usernode.setNodeValue("disabled", "false");
    usernode.setNodeValue("id", user.getId());

    Properties attrs = user.getAttributes();
    if (attrs != null) {
      Enumeration en = attrs.propertyNames();
      while (en.hasMoreElements()) {
        try {
          String attrName = (String) en.nextElement();
          String attrValue = attrs.getProperty(attrName);
          usernode.setNodeValue(attrName, attrValue);
        } catch (Exception e) {
        }
      }
    }
  }
}
