/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.security;

import java.util.Enumeration;
import java.util.Properties;
import org.moremotion.datasrc.DataService;
import org.moremotion.datasrc.DataServiceContext;

import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.Config;
import org.moremotion.datasrc.DataSourceConfig;
import org.moremotion.security.CheckPoint;
import org.moremotion.security.SecurityContext;
import org.moremotion.security.SecurityContextException;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: CheckPointList.java 51 2008-03-10 11:46:54Z erkan $
 */
public class CheckPointList implements DataService {

  public void doService(ADOM adom, DataServiceContext dsc) throws ServiceException {
    
    try {
      
      adom.clear();

      MoreMotionRequest request = dsc.getRequest();
      DataSourceConfig dsconfig = dsc.getDataSourceConfig();
      String secdom = Utils.getSecurityDomainName(request, dsconfig);
      Config sdconfig = Utils.getSecurityDomainConfig(secdom);
      SecurityContext sctx = Utils.getSecurityContext(request, sdconfig);

      CheckPoint[] checkPoints = sctx.getCheckPoints();
      for (int i = 0; i < checkPoints.length; i++) {
        CheckPoint checkPoint = checkPoints[i];
        ADOMNode node = new ADOMNode("item");
        node.setNodeValue("id", checkPoint.getId());
        node.setNodeValue("name", checkPoint.getName());
        node.setNodeValue("desc", checkPoint.getDescription());

        Properties attrs = checkPoint.getAttributes();
        if (attrs != null) {
          Enumeration en = attrs.propertyNames();
          while (en.hasMoreElements()) {
            String attrName = (String) en.nextElement();
            String attrValue = safeval(attrs.getProperty(attrName));
            node.setNodeValue(attrName, attrValue);
          }
        }

        adom.addNode(node);
      }
    } catch (SecurityContextException ex) {
      throw new ServiceException(ex);
      
    } catch (SecurityException ex) {
      throw new ServiceException(ex);
      
    }
    
  }

  private String safeval(String s) {
    if (s == null) {
      return "";
    }
    return s;
  }
}
