/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * HasAuthorityMScriptFunction.java
 *
 * Created on 23 September 2007 Sunday, 14:43
 *
 */

package org.moremotion.mmcomp.security;

import org.moremotion.config.Config;
import org.moremotion.evaluator.EvaluationException;
import org.moremotion.evaluator.Expression;
import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.evaluator.MScriptFunction;
import org.moremotion.evaluator.MScriptFunctionContext;
import org.moremotion.evaluator.MScriptFunctionException;
import org.moremotion.security.SecurityContext;
import org.moremotion.security.SecurityContextException;
import org.moremotion.security.User;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: HasAuthorityMScriptFunction.java 190 2008-08-09 16:19:03Z erkan $
 */
public class HasAuthorityMScriptFunction implements MScriptFunction {
  
  public Object execute(Expression[] params, MScriptFunctionContext msfc) throws MScriptFunctionException {
    
    if (params.length < 1) {
      throw new MScriptFunctionException("LESS_PARAMETERS_THAN_EXPECTED",new String[]{"hasAuthority()"},null);
    }
    
    try {
      
      MoreMotionRequest request = msfc.getRequest();
      MMSymbolResolver sr = msfc.getSymbolResolver();
      
      String secdom = "main";
      if (params.length > 1) {
        secdom = params[1].evaluate(sr).toString();
      }
      if (secdom.trim().length() == 0) secdom = "1";
      
      Config sdconfig = Utils.getSecurityDomainConfig(secdom);
      SecurityContext sctx = Utils.getSecurityContext(request, sdconfig);
      
      User user = sctx.getCurrentUser();
      if (user != null) {
        String cpname = params[0].eval(sr);
        return new Boolean(sctx.hasAuthority(user, cpname));
      }
      return new Boolean(false);
      
    } catch (SecurityException ex) {
      throw new MScriptFunctionException(ex);
      
    } catch (EvaluationException ex) {
      throw new MScriptFunctionException(ex);
      
    } catch (SecurityContextException ex) {
      throw new MScriptFunctionException(ex);
    }
    
  }
  
}
