/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.security;

/*
 * SMLProcess.java
 *
 * Created on February 10, 2003, 1:04 PM
 */

import org.moremotion.config.Config;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigurationException;
import org.moremotion.security.SecurityContextException;
import org.moremotion.process.ProcessException;
import org.moremotion.process.Process;
import org.moremotion.process.ProcessRecord;
import org.moremotion.security.SecurityContext;
import org.moremotion.security.User;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SMLProcess.java 51 2008-03-10 11:46:54Z erkan $
 */
public class SMLProcess extends Process {
  
  private static final int ADMIN = 1;
  private static final int LUSER = 2;
  
  private ConfigParameter authority;
  private ConfigParameter checkpoint;
  private String smlStr;
  private String secdom;
  private SML sml;
  
  
  public void prepare() throws ProcessException {
    
    if (logger.LEVEL1) {
      logger.printres("*PROCESS_STARTED", callDef.getStepName());
    }
    
    try {
      
      secdom = processConfig.getParameter("secdom").resolve(request).stringValue("main");
      Config sdconfig = Utils.getSecurityDomainConfig(secdom);
      SecurityContext sctx = Utils.getSecurityContext(request, sdconfig);
      
      authority = processConfig.getParameter("authority");
      checkpoint = processConfig.getParameter("checkpoint");
      
      
      smlStr = processConfig.getParameter("sml").stringValue();
      
      if (logger.LEVEL2) {
        logger.printres("*PROCESS_PARAMETERS_LINE");
        logger.printres("SML_PROCESS_PARAMETERS", authority+"", checkpoint+"", smlStr+"");
      }
      
      String Checkpoint = checkpoint.resolve(request).stringValue("");
      if (logger.LEVEL2 && checkpoint.hasMScript()) {
        logger.printres("CHECKPOINT_RESOLVED", Checkpoint);
      }
      
      int Authority = authority.resolve(request).intValue(LUSER);
      
      
      
      
      User currentUser = sctx.getCurrentUser();
      
      if (Authority == ADMIN || Checkpoint.equals("") || sctx.hasAuthority(currentUser, Checkpoint) ) {
        if (logger.LEVEL2) logger.printres("AUTHORITY_SUFFICIENT_FOR_SML");
      } else {
        if (logger.LEVEL1) logger.printres("AUTHORITY_INSUFFICIENT_FOR_SML");
        throw new ProcessException(this,"INSUFFICIENT_AUTHORITY_TO_EXECUTE_SML",null,null);
      }
      
      sml = new SML(sctx, sdconfig, request, logger);
      
    } catch (SecurityException ex) {
      throw new ProcessException(ex);
      
    } catch (ConfigurationException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
      
    } catch (SecurityContextException ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(ex);
    }
    
  }
  
  public void processRecord(ProcessRecord prec) throws ProcessException {
    
    try {
      setCompletionCode( sml.execute(smlStr) );
      
    } catch(Exception ex) {
      if (logger.LEVEL1) logger.printres("*EXCEPTION_OCCURED", ex.getMessage());
      throw new ProcessException(this, "SML_PROCESS_ERROR",null,ex);
    }
  }
  
  public void finalizeProcess() throws ProcessException {
    if (logger.LEVEL2) {
      logger.printres("*PROCESS_FINALIZING",callDef.getStepName());
    }
  }
  
}
