/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.mmcomp.security;

import org.moremotion.datasrc.DataService;
import org.moremotion.datasrc.DataServiceContext;
import org.moremotion.security.RememberUser;
import org.moremotion.security.SecurityContextException;
import org.moremotion.adom.ADOM;
import org.moremotion.config.Config;

import org.moremotion.security.SecurityContext;
import org.moremotion.security.User;
import org.moremotion.servlet.ServiceException;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: UserInfo.java 226 2008-10-06 14:51:47Z erkan $
 */
public class UserInfo implements DataService {
  
  /** Generates <CODE>mor_security_userinfo</CODE> ADOM.
   * @param adom The ADOM object to be filled with data.
   * @param dsc DataServiceContext object
   */
  public void doService(ADOM adom, DataServiceContext dsc) throws ServiceException {
    
    try {
      
      MoreMotionRequest request = dsc.getRequest();
      String secdom = Utils.getSecurityDomainName(request, dsc.getDataSourceConfig());
      Config sdconfig = Utils.getSecurityDomainConfig(secdom);
      SecurityContext sctx = Utils.getSecurityContext(request, sdconfig);
      
      User user = sctx.getCurrentUser();
      if (user != null) {
        
        boolean populate = adom.getNodeCount() == 0;
        if (!populate) {
          String lm = "" + user.getLModified();
          try {
            if (!adom.getAttribute("lmodified").equals(lm)) {
              populate = true;
            }
          } catch (Exception ex) {
            populate = true;
          }
        }
        if (populate) {
          adom.clear();
          Utils.populateUserInfo(adom, user, true);
        }
        
      } else {
        
        RememberUser remember = sctx.getRememberUser();
        
        if (remember.getOption() == 1) {
          adom.clear();
          return;
        }
        
        adom.clear();
        adom.setNodeValue("username",remember.getRecentUserName());
        adom.setNodeValue("subdomain",remember.getRecentSubDomain());
      }
      
    } catch (SecurityContextException ex) {
      throw new ServiceException(ex);
      
    } catch (SecurityException ex) {
      throw new ServiceException(ex);
    }
  }
  
}
