/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Utils.java
 *
 * Created on July 16, 2006, 2:43 PM
 */

package org.moremotion.mmcomp.security;

import java.util.Enumeration;
import java.util.Properties;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigurationException;
import org.moremotion.security.SecurityManager;

import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigManager;
import org.moremotion.security.SecurityContext;
import org.moremotion.security.SecurityContextException;
import org.moremotion.security.User;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.SystemConfig;
import org.moremotion.servlet.SystemContext;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Utils.java 51 2008-03-10 11:46:54Z erkan $
 */
public class Utils {
  
  protected static void populateUserInfo(ADOM uinfo, User user, boolean loggedIn) {
    _populateUserInfoADOM(uinfo, user, loggedIn);
  }
  
  private static void _populateUserInfoADOM(ADOM uinfo, User user, boolean loggedIn) {
    
    uinfo.setNodeValue("id", "" + user.getId());
    uinfo.setNodeValue("username", user.getName());
    uinfo.setNodeValue("email", user.getEmail());
    uinfo.setNodeValue("fullname", user.getFullname());
    
    uinfo.addAttribute("lmodified", "" + user.getLModified());  
    
    Properties userAttributes = user.getAttributes();
    if (userAttributes != null) {
      Enumeration e = userAttributes.keys();
      while (e.hasMoreElements()) {
        String name = (String)e.nextElement();
        String value = userAttributes.getProperty(name);
        uinfo.setNodeValue(name, value);
      }
    }
    
    if (loggedIn) {
      uinfo.setNodeValue("loggedin", "true");
      uinfo.setNodeValue("lmodified", user.getLModified() + "");
      uinfo.setNodeValue("signIn", "" + user.getSignInTime());  
      uinfo.setNodeValue("lastAccess", "" + user.getLastReferredTime());
      
      String[] cps = user.getCheckPointNames();
      if (cps != null) {
        ADOMNode checkpoints = new ADOMNode("checkpoints");
        uinfo.addNode(checkpoints);
        
        for (int i = 0; i < cps.length; i++) {
          String cpname = cps[i];
          if (org.moremotion.evaluator.Conventions.isValidEntityIdentifier(cpname)) { 
            ADOMNode cpnode = new ADOMNode(cpname,"true");
            checkpoints.addNode(cpnode);
            // checkpoints.setNodeValue(cpname,"true");
          }
        }

        for (int i = 0; i < cps.length; i++) {
          ADOMNode node = new ADOMNode("checkpoint",cps[i]);
          checkpoints.addNode(node);
        }

      }
    }
  }
  
  public static Config getSecurityDomainConfig(String secdom) throws SecurityException {
    try {
      Config config = ConfigManager.getConfig("securityDomain", secdom);
      if (config == null) {
        throw new SecurityException("SECURITY_DOMAIN_CONFIGURATION_MISSING", new String[]{secdom}, null);
      }
      return config;
      
    } catch (ConfigurationException ex) {
      throw new SecurityException(ex);
    }
  } 
  
  public static String getSecurityDomainName(MoreMotionRequest request, Config config) throws SecurityException {
    try {
      String secdom = config.getParameter("secdom").resolve(request).stringValue("main");
      return secdom;
    } catch (ConfigParameterException ex) {
      throw new SecurityException(ex);
    }
  }
  
  
  public static SecurityContext getSecurityContext(MoreMotionRequest request, Config sdconfig) throws SecurityContextException {
    
    try {
      String securityDomain = sdconfig.getName();
      return SecurityManager.getSecurityContext(request, securityDomain );
      
    } catch (Exception e) {
      throw new SecurityContextException(e);
    }
  }
  
  public static String getAccessUrl(MoreMotionRequest request, String secdom, User user, String subdomain, String accessPage, String enckey, boolean temporary) throws Exception {
    
    StringBuffer url = new StringBuffer();
    url.append(request.getApplicationPath() + "/SignIn." + SystemConfig.actionServiceExtension());
    
    if (subdomain == null) subdomain = "";
    
    String temp = temporary ? "&temporary=true" : "";
    String key = 
        "username=" + user.getName() + 
        "&subdomain=" + subdomain + 
        "&nextpage=" + accessPage + 
        "&password=" + user.getPassword() + temp + 
        "&_rand=" + System.currentTimeMillis();
    key = SystemContext.encryptor().encrypt(key,"accurl" + enckey);

    url.append("?key=" + key);
    url.append("&secdom=" + secdom);
    
    return url.toString();
  }
  
  
  
}