/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.adom;


import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.Cookie;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ADOMUtils.java 51 2008-03-10 11:46:54Z erkan $
 */
public final class ADOMUtils {
  
  public static String getNodePath(String str) {
    
    String result = XString.after(str,':');
    result = result.replace('/','.');
    if (result.charAt(0) == '.') {
      result = result.substring(1);
    }
    return XString.after(result,'.',null);
  }
  
  /**
   * Extracts the ADOM Name from the given nodePath definition and checkes if an ADOM is associated
   * with it. The search is done in request, session, application order.
   * <P><B>Examples:</B><BR/>
   * <CODE>s:/cart</CODE> : The scope is given as "s" so ADOM <B>cart</B> will be searched in session scope.<BR/>
   * <CODE>/myds/item</CODE> : The scope is not given so ADOM <B>myds</B> will be searched in all scopes.<BR/>
   * </PRE>
   * 
   * @param request MoreMotionRequest object
   * @param nodePath The node path definition that contains the ADOM name.
   * @return The ADOM found. If No ADOM is found <CODE>null</CODE> is returned.
   */
  public static ADOM findADOM(MoreMotionRequest request, String nodePath) {
    
    ADOM adom = null;
    
    String prefix = XString.before(nodePath, ':', "");
    String adomName = XString.after(nodePath, ':').trim();
    if (adomName.length() == 0) return null;
    if (adomName.charAt(0) == '/') adomName = adomName.substring(1);
    int x = adomName.indexOf('/');
    if (x > -1) adomName = adomName.substring(0,x);
    adomName = "ADOM_" + adomName;
    
    char scope = ' ';
    for (int i = 0; i < prefix.length();i ++) {
      char c = prefix.charAt(i);
      if (c == 'r' || c == 's' || c == 'a') {
        scope = c;
        break;
      }
    }
    
    if (scope == 'r') {
      adom = (ADOM) request.getAttribute(adomName);
      
    } else if (scope == 's') {
      adom = (ADOM) request.getSession().getAttribute(adomName);
      
    }  if (scope == 'a') {
      adom = (ADOM) request.getContext().getAttribute(adomName);
      
    } if (scope == ' ') {
      adom = (ADOM) request.getAttribute(adomName);
      if (adom == null) {
        adom = (ADOM) request.getSession().getAttribute(adomName);
        if (adom == null) {
          adom = (ADOM) request.getContext().getAttribute(adomName);
        }
      }
    }
    
    return adom;
  }
  
 
  /**
   * Extracts the name of the ADOM from the given node path definition and returns it.
   */
  public static String getPureADOMName(String nodePath) {
    
    String result = XString.after(nodePath,':');
    result = result.replace('/','.');
    if (result.charAt(0) == '.') {
      result = result.substring(1);
    }
    return XString.before(result,'.');
  }
  
  
  /**
   * Extracts the scope identifier from the given node path definition and returns it.
   */
  public static char getADOMScope(String nodePath) {
    
    String prefix = XString.before(nodePath,':', " ");
    
    if (prefix.indexOf('r') > -1 ) return 'r';
    else if (prefix.indexOf('s') > -1 ) return 's';
    else if (prefix.indexOf('a') > -1 ) return 'a';
    else return ' ';
    
  }
  
  
}