/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * SortComparator.java
 *
 * Created on 06 August 2006 Sunday, 02:45
 */

package org.moremotion.adom;

import java.util.Comparator;
import java.util.LinkedList;
import org.moremotion.util.StringParser;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SortComparator.java 51 2008-03-10 11:46:54Z erkan $
 */
public class SortComparator implements Comparator {
  
  SortParam[] sortparams;
  
  public SortComparator(String sortDefinition) {
    parseSortParams(sortDefinition);
  }
  
  private void parseSortParams(String params) {
    
    LinkedList ll = new LinkedList();
    StringParser parser = new StringParser(params,';');
    while (parser.hasMore()) {
      String param = parser.nextItem();
      
      StringParser par = new StringParser(param,',');
      String nn = par.nextItem();
      char order = " desc d 2 ".indexOf(" " + par.nextItem() + " ") > -1 ? 'd' : 'a';
      
      boolean isnum = " true True TRUE 1 ".indexOf(" " + par.nextItem() + " ") > -1 ? true : false;
      
      SortParam sparam = new SortParam(nn,order,isnum);
      ll.addLast(sparam);
      
    }
    
    sortparams = new SortParam[ll.size()];
    ll.toArray(sortparams);
    
  }
  
  
  public int compare(Object o1, Object o2) {
    return _compare((ADOMNode)o1,(ADOMNode)o2,0);
  }
  
  private int _compare(ADOMNode item1, ADOMNode item2, int index) {
  
    if (index < 0 || index > sortparams.length) return 0;
    
    SortParam sparam = sortparams[index];
    
    int result = 0;
    
    ADOMNode n1 = item1.findNode(sparam.nodeName);
    ADOMNode n2 = item2.findNode(sparam.nodeName);
    
    if (sparam.isnum) {
      double p1 = n1 == null ? 0 : n1.getValueAsDouble();
      double p2 = n2 == null ? 0 : n2.getValueAsDouble();
      if (p1 == p2) result = 0;
      else {
        if (sparam.order == 'a') {
          result = p1 > p2 ? 1 : -1;
        } else {
          result = p2 > p1 ? 1 : -1;
        }
      }
    } else {
      String p1 = n1 == null ? "" : n1.getValue();
      String p2 = n2 == null ? "" : n2.getValue();
      if (sparam.order == 'a') {
        result = p1.compareTo(p2);
      } else {
        result = p2.compareTo(p1);
      }
    }
    
    if (result == 0 && index+1 < sortparams.length) {
      return _compare(item1,item2,index+1);
    } 
    return result;
    
  }
  
  
  class SortParam {
    
    String nodeName;
    char order;
    boolean isnum;
    
    SortParam(String nodeName, char order, boolean isnum) { 
      this.nodeName = nodeName;
      this.order = order;
      this.isnum = isnum;
    }
    
  }
  
  
}


