/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * CompiledObjectCache.java
 *
 * Created on 01 November 2005 Tuesday, 19:39
 */

package org.moremotion.cache;

import java.io.File;
import java.util.Enumeration;
import java.util.Hashtable;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: CompiledObjectCache.java 51 2008-03-10 11:46:54Z erkan $
 */
public class CompiledObjectCache implements FileChangeNotification {
  
  private Hashtable cache = new Hashtable();
  private int capacity = 5;
  
  public void setCapacity(int value) {
    this.capacity = value;
  }
  
  public Object getCompiledObject(File file) {
    return get( file.getAbsolutePath() );
  }
  
  
  public void addCompiledObject(File file, Object co) {
    
    synchronized (cache) {
      
      if (cache.size() > capacity) {
        removeLeastImportant();
      }
      
      COEntry coe = new COEntry(file,co);
      cache.put(file.getAbsolutePath(), coe);
      
      FileChangeDetector cm = FileChangeDetector.getInstance();
      cm.addFile(file, this);
    }
    
    
  }
  
  public void doChange(File f) {
    remove(f);
  }
  
  /**
   * Remove the least important Compile Object from the cache.
   * Currently this method only evaluates the popularity of the objects. 
   * TODO: Implement other criterias such as object size and recent usages.
   */
  private void removeLeastImportant() {
    File f = null;
    long lu = 9999999;
    Enumeration e = cache.elements();
    while (e.hasMoreElements()) {
      COEntry coe = (COEntry)e.nextElement();
      if (coe.popularity < lu) {
        f = coe.file;
        lu = coe.popularity;
      }
    }
    remove(f);
  }
  
  
  private Object get(String key) {
    COEntry coe = (COEntry)cache.get(key);
    if (coe == null) return null;
    synchronized (this) {
      coe.touch();
    }
    return coe.co;
  }
  
  private void remove(File file) {
    synchronized (this) {
      cache.remove(file.getAbsolutePath());
    }
  }
  
  public void clear() {
    cache = new Hashtable();
  }

  public void doDelete(File f) {
  }
  
}

class COEntry {
  protected File file;
  protected long popularity;
  protected long lastUsed;
  protected Object co;
  protected COEntry(File file, Object co) {
    this.file = file;
    this.co = co;
    this.popularity = 0;
    this.lastUsed = System.currentTimeMillis();
  }
  protected void touch() {
    this.popularity++;
    this.lastUsed = System.currentTimeMillis();
  }
}
