/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * VariablePool.java
 *
 * @author Erkan Cemgiz
 */

package org.moremotion.cache;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;



/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: VariablePool.java 51 2008-03-10 11:46:54Z erkan $
 */
public class VariablePool {  
  
  private static final String STACK_DELIMITER = "^stack^-^delimiter^";
  private static final String NULL_INDICATOR = "^null^-^indicator^";
  private Properties vpool = null;
  private Hashtable opool = null;
   
  /** Creates a new instance of VariablePool */
  public VariablePool() {
    vpool = new Properties();
    opool = new Hashtable();
  }
  
  public String get(String varName) {
    String result = vpool.getProperty(varName);
    if (result != null) {
      int i = result.indexOf(STACK_DELIMITER);
      if (i > -1) {
        result = result.substring(0,i);
      }
    }
    return result;
  }
  
  public String get(String varName, String defaultValue) {
    String result = get(varName);
    if (result == null) result = defaultValue;
    return result;
  }
  
  public void put(String varName, String value) {
    vpool.setProperty(varName, value);
  }
  
  
  public Object getObject(String key) {
    return opool.get(key);
  }
  
  public void putObject(String key, Object object) {
    opool.put(key,  object);
  }
  
  public void removeObject(String key) {
    opool.remove(key);
  }
  
  
  public Enumeration getNames() {
    return vpool.keys();
  }
  
  public boolean has(String varName) {
    return vpool.getProperty(varName) != null;
  }
  
  public void remove(String varName) {
    vpool.remove(varName);
  }
  
  public void push(String varName, String value) {
    String s = vpool.getProperty(varName);
    if (s == null) s = NULL_INDICATOR;
    s = value + STACK_DELIMITER + s;
    vpool.setProperty(varName, s);
  }
  
  public String pop(String varName) {
    String result = vpool.getProperty(varName);
    if (result == null) return null;
    int i = result.indexOf(STACK_DELIMITER);
    if (i > -1) {
      String s = result.substring(i + STACK_DELIMITER.length());
      result = result.substring(0,i);
      if (s.equals(NULL_INDICATOR)) {
        vpool.remove(varName);
      } else {
        vpool.setProperty(varName, s);
      }
    } else {
      vpool.remove(varName);
    }
    return result;
  }
  
  public String pop(String varName, String defaultValue) {
    String result = pop(varName);
    if (result == null) {
      result = defaultValue;
    }
    return result;
  }
  
  
  /**
   * Removes the entries whose variable names start with the specified prefix.
   */
  public void removeGeneric(String namePrefix) {
    Enumeration e = vpool.keys();
    while (e.hasMoreElements()) {
      String s = (String)e.nextElement();
      if (s.startsWith(namePrefix)) {
        vpool.remove(s);
      }
    }
    
  }
  
  /**
   * Creates a snap-shot of this Variable pool as Properties object and returns it.
   */
  public Properties getSnapShot() {
    Properties result = new Properties();
    Enumeration e = vpool.keys();
    while (e.hasMoreElements()) {
      String n = (String)e.nextElement();
      String v = get(n);
      if (v != null) {
        result.setProperty( new String(n), new String(v) ) ;
      }
    }
    return result;
  }
  
}
