/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.cache;

import java.io.File;
import java.io.PrintWriter;
import java.io.StringReader;
import java.io.StringWriter;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Templates;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMResult;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import org.w3c.dom.Document;
import org.w3c.dom.Node;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: XslTransformer.java 51 2008-03-10 11:46:54Z erkan $
 */

public class XslTransformer {
  
  private Templates templates;
  
  public XslTransformer(File xsl) throws Exception {
    TransformerFactory factory = TransformerFactory.newInstance();
    templates = factory.newTemplates(new StreamSource(xsl));
  }
  
  public XslTransformer(String xsl) throws Exception {
    TransformerFactory factory = TransformerFactory.newInstance();
    StringReader sr = new StringReader(xsl);
    templates = factory.newTemplates(new StreamSource(sr));
  }
  
 
  public String transform(String xml) throws TransformerConfigurationException, TransformerException {
    String result = null;
    StringReader src = new StringReader(xml);
    StringWriter sink = new StringWriter();
    templates.newTransformer().transform(new StreamSource(src), new StreamResult(sink));
    result = sink.toString();
    return result;
  }

  public Node transformToDom(String xml) throws TransformerConfigurationException, TransformerException {
    StringReader src = new StringReader(xml);
    DOMResult result = new DOMResult();
    Transformer transformer = templates.newTransformer();
    transformer.setOutputProperty(OutputKeys.METHOD,"xml");
    transformer.transform(new StreamSource(src), result);
    return result.getNode();
  }

  /*
  public String transform(File xml) {
    String result = null;
    try {
      StringWriter sink = new StringWriter();
      templates.newTransformer().transform(new StreamSource(xml), new StreamResult(sink));
      
      result = sink.toString();
    } catch(TransformerConfigurationException tce) {
      result = stackTrace(tce);
    } catch(TransformerException te) {
      result = stackTrace(te);
    }
    return result;
  }
  */
  
  public void transform(Document xml, Node node) throws TransformerConfigurationException, TransformerException {
    
    DOMResult domResult = new DOMResult(node);
    templates.newTransformer().transform(new DOMSource(xml), domResult);
    
  }
  
  private static String stackTrace(Exception e) {
    StringWriter sw = new StringWriter();
    PrintWriter pw = new PrintWriter(sw);
    pw.println("<PRE>");
    e.printStackTrace(pw);
    pw.println("</PRE>");
    pw.flush();
    return sw.toString();
  }
}