/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ConditionalList.java
 */

package org.moremotion.config;

import java.util.ArrayList;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ConditionalList.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ConditionalList {
  
  ArrayList listItems = new ArrayList();
  
  /** Creates a new instance of ConditionalList */
  public ConditionalList() {
  }
  
  public void add(ConditionalListItem listItem) {
    listItems.add(listItem);
  }
  
  /**
   * Returns a ConditionalListIterator to iterate over the ConditionalListItems contained
   * @param request The MoreMotionRequest object
   */
  public ConditionalListIterator iterator(MoreMotionRequest request) {
    return new ConditionalListIterator(this,request);
  }
  
  /**
   * Returns a ConditionalListItem array after evaluating the conditions
   * @param request The MoreMotionRequest object
   */
  public ConditionalListItem[] getItems(MoreMotionRequest request) throws ConfigParameterException  {
    ArrayList a = new ArrayList();
    ConditionalListIterator itr = iterator(request);
    while (itr.hasMore()) {
      a.add(itr.next());
    }
    
    ConditionalListItem[] result = new ConditionalListItem[a.size()];
    a.toArray(result);
    
    return result;
  }
  
  protected int nextItemIndex(MoreMotionRequest request, int currIndex) throws ConfigParameterException {
    if (currIndex >= listItems.size()) {
      return -1;
    }
    ConditionalListItem cli = (ConditionalListItem)listItems.get(currIndex);
    if (cli.precondSatisfied(request)) {
      return currIndex;
    } else {
      return nextItemIndex(request, currIndex + 1);
    }
  }
  
  protected ConditionalListItem get(int currIndex) {
    return (ConditionalListItem)listItems.get(currIndex);
  }
  
  public ArrayList getList() {
    return listItems;
  }
  
  
}
