/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Config.java
 *
 */

package org.moremotion.config;

import java.util.HashMap;

/**
 * Base Class for Config Classes
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Config.java 51 2008-03-10 11:46:54Z erkan $
 */
public class Config {

  private String name;
  private Object data = null;
  private ConfigUnit configUnit = null;
  protected HashMap params = null;
  
  public Config(ConfigUnit unit, boolean parse) throws ConfigurationException {
    this.configUnit = unit;
    this.name = unit.getName();
    if (parse) parse(unit.getConfigNodeList());
  } 
  
//  public Config(String name, ConfigNodeList nodelist, boolean parse) throws ConfigurationException {
//    this.name = name;
//    if (parse) parse(nodelist);
//  } 
  
  public String getName() {
    return name;
  }
  
 public ConfigUnit getConfigUnit() {
   return this.configUnit;
 }
  
  public void setDataObject(Object data) {
    this.data = data;
  }
  
  public Object getDataObject() {
    return this.data;
  }
  
  private void parse(ConfigNodeList nodelist) throws ConfigParameterException {
    
    for (int i = 0; i < nodelist.size(); i++) {
      
      ConfigNode node = nodelist.get(i);
      
      if (params == null) params = new HashMap();
      params.put(node.getName(), new ConfigParameter(node.getName(), node.getValue() )) ;
      
    }
    
  }
  
  /**
   * Returns the value of the specified special parameter
   */
  public ConfigParameter getParameter(String name) throws ConfigParameterException {
    if (params == null) {
      return new ConfigParameter(name,null);
    }
    ConfigParameter param = (ConfigParameter)params.get(name);
    return param == null ? new ConfigParameter(name,null) : param;
  }
  
  /**
   * Returns the special parameters (non-standard) of the configuration
   */
  public HashMap getParameters() {
    return this.params;
  }
  
  
  
  
}
