/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * DataSourceManager.java
 *
 */

package org.moremotion.datasrc;

import java.util.HashMap;
import org.moremotion.adom.ADOM;
import org.moremotion.config.ConfigCache;
// import org.moremotion.config.ConfigNodeList;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.servlet.ServiceException;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: DataSourceManager.java 115 2008-04-08 11:56:01Z erkan $
 */
public class DataSourceManager {
  
  private static HashMap serviceCach = new HashMap();
  
  private DataSourceManager() {}
  
  public static DataSourceConfig getDataSourceConfig(MoreMotionRequest request, String name) throws ConfigurationException {
    
    String originPage = request.getOriginPage();
    
    ConfigUnit unit = ConfigCache.getUnit("dataSource", originPage + '.' + name);
    if (unit == null) {
      unit = ConfigCache.getUnit("dataSource", name);
    }
    
    if (unit == null) {
      return null;
    }
    
//    if (unit == null) {
//      throw new ConfigurationException("CONFIG_UNIT_MISSING",new String[]{name},null);
//    }
    
    if (unit.isNew()) {
      synchronized (unit.getSyncObject()) {
        parse(unit);
      }
    }
    
    return (DataSourceConfig)unit.getDataObject();
  }

  
//  public static Paging getPaging(String dsname, MoreMotionRequest request) throws ServiceException {
//    try {
//      DataSourceConfig dsconfig = getDataSourceConfig(dsname);
//      if (dsconfig == null) {
//        return null;
//      }
//      return new Paging(request, dsconfig);
//    } catch (Exception e) {
//      throw new ServiceException(e);
//    }
//  }
  
  private static void parse(ConfigUnit unit) throws ConfigurationException {
    
    // ConfigNodeList nodelist = unit.getConfigNodeList();
    
    DataSourceConfig dsconfig = new DataSourceConfig(unit);
    // DataSourceConfig dsconfig = new DataSourceConfig(unit.getName(), nodelist);
    unit.setDataObject(dsconfig);
    
  }
  
  
  public static void runDataService(String serviceName, ADOM adom, DataServiceContext dsc) throws ServiceException {
    
    DataService service = getServiceClass(serviceName);
    service.doService(adom, dsc);
    
  }
  
  private static DataService getServiceClass(String className) throws ServiceException {
    
    Object object = serviceCach.get(className);
    
    if (object == null) {
      
      synchronized (serviceCach) {
        
        object = serviceCach.get(className);
        if (object == null) {
          
          try {
            object = (Object) Class.forName(className).newInstance();
          } catch (Exception e) {
            throw new ServiceException("DATA_SERVICE_CLASS_MISSING",new String[]{className},e);
          }
          
          serviceCach.put(className, object);
        }
        
      }
    }
    
    return (DataService)object;
  }
  
}
