/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */


package org.moremotion.datasrc;

import org.moremotion.config.ConfigurationException;
import org.moremotion.page.PageGenerationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.StringParser;
import org.moremotion.util.XString;


/**
 * Changes current page of a specified datasource.
 * @author  Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: RefreshDataSource.java 51 2008-03-10 11:46:54Z erkan $
 */
public class RefreshDataSource {
  
  private static int getInteger(String strVal, int defVal) {
    
    int result = defVal;
    try {
      result = Integer.parseInt(strVal);
    } catch (Exception e) {}
    
    return result;
  }
  
  
  public static void refresh(MoreMotionRequest request) throws PageGenerationException {
    
    String[] defs = request.getParameterValues("_refreshds");
    
    if (defs == null || defs.length == 0) return;
    
    for (int i = 0; i < defs.length; i++) {
      
      String def = defs[i];
      
      String ds = null;
      String cpStr = null;
      String ippStr = null;
      String ppbStr = null;
      
      StringParser parser = new StringParser(def,';');
      while (parser.hasMore()) {
        String s = parser.nextItem();
        String n = XString.before(s,':',null);
        String v = XString.after(s,':',null);
        if (n != null && v != null) {
          if (n.equals("ds")) ds = v; 
          else if (n.equals("cp")) cpStr = v; 
          else if (n.equals("ipp")) ippStr = v; 
          else if (n.equals("ppb")) ppbStr = v; 
        }
      }
      
      if (ds == null || ds.length() == 0) continue;
      
      request.setAttribute("moremotion.refresh.datasource." + ds ,"true");
      
      int cp = getInteger(cpStr, -2);
      int ipp = getInteger(ippStr, -1);
      int ppb = getInteger(ppbStr, -1);
      
      try {
        process(request, ds, cp, ipp, ppb);
      } catch (ConfigurationException ex) {
        throw new PageGenerationException(ex);
      }
    }
    
  }
  
  
  private static void process(MoreMotionRequest request, String datasource, int cp, int ipp, int ppb) throws ConfigurationException {
    
    DataSourceConfig dsconfig = DataSourceManager.getDataSourceConfig(request, datasource);
    
    if (dsconfig != null) {
      
      String adomName = datasource;
      
      Paging paging = new Paging(request, adomName, dsconfig);
      paging.setNeedsRefresh(true);
      
      if (ipp > 0) {
        paging.setItemsPerPage(ipp);
      }
        
      if (ppb > 0) {
        paging.setPagesPerBlock(ppb);
      }
      
      if (cp == -1) cp = 999999;
      if (cp > 0) {
        paging.setCurrentPage(cp);
      }
      
    }
    
  }
  
}
