/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * EvalUtils.java
 *
 * Created on December 21, 2001, 10:36 AM
 */

package org.moremotion.evaluator;

import java.util.Properties;
import java.util.StringTokenizer;
import java.util.GregorianCalendar;
/**
 * @author Mehmet Tastan
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: EvalUtils.java 61 2008-03-14 14:56:20Z erkan $
 */
public class EvalUtils {
  
  private static final String DLM = " '\"+-%(),;*=";
  private static java.text.DecimalFormat decimalFormatter = new java.text.DecimalFormat();
  private static java.text.SimpleDateFormat dateFormatter = new java.text.SimpleDateFormat("yyyy.MM.dd HH:mm:ss");
  
  static {
    decimalFormatter.applyPattern("###.###");
  }
  
  public static boolean looksLikeSymbol(String s) {
    if (s.length() == 0) return false;
    char c = s.charAt(0);
    if (c != '.' && Symbol.SYMBOL_VALID_FIRST_CHARS.indexOf(c) == -1) return false;
    for (int i = 1; i < s.length(); i++) {
      c = s.charAt(i);
      if (DLM.indexOf(c) > -1) return false;
    }
    return true;
  } 
  
  public static boolean isSymbol(String s) {
    Symbol sym = new Symbol(s);
    return sym.getError() == null;
  }
  
  public static boolean isADOMName(String s) {
    int ln = s.length();
    if (ln == 0) return false;
    
    if (Symbol.SYMBOL_VALID_FIRST_CHARS.indexOf(s.charAt(0)) == -1) return false;
    
    if (ln > 1) {
      int start;
      if (s.charAt(1) == ':') {
        if (ln == 2) return false;
        char c = s.charAt(0);
        if (c != 'r' && c != 's' && c != 'a') return false;
        start = 2;
      } else {
        start = 1;
      }
      
      for (int i = start; i < ln; i++) {
        if ( Symbol.SYMBOL_VALID_BODY_CHARS.indexOf( s.charAt(i) ) == -1 ) return false;
      }
    }
    
    return true;
    
  }
  
  public static boolean isString(String str) {
    
    if (str == null || str.length() < 2 || str.charAt(0) != '\'' || str.charAt(str.length()-1) != '\'') {
      return false;
    }
    
    return true;
  }
  
  public static double getNumber(String s) {
    try {
      return Double.parseDouble(s);
    } catch (Exception ignore){}
    return 0;
  }
  
  public static boolean getBoolean(String s) {
    return s != null && s.equalsIgnoreCase("true");
  }
  
  public static boolean isBoolean(String str) {
    return (str != null) && (str.equalsIgnoreCase("true") || str.equalsIgnoreCase("false"));
  }
  
  public static Double doubleValue(Object o) {
    if (o == null) return new Double(0);
    if (o instanceof Double) return (Double)o;
    String s = EvalUtils.toStr(o);
    if (s.length() == 0)  return new Double(0);
    return new Double( s );
  }
  
  public static boolean isNumber(String str) {
    if (str == null || str.length() == 0) return false;
    boolean dot = false;
    char c = str.charAt(0);
    int start = c == '-' || c == '+' ? 1 : 0;
    for (int i = start; i < str.length(); i++) {
      c = str.charAt(i);
      if (c == '.') {
        if (dot) {
          return false;
        }
        dot = true;
      } else {
        if (c < 48 || c > 57) {
          return false;
        }
      }
    }
    return true;
  }
  
  
  public static String getValueOfStringConstant(String str, char quote) throws InvalidEntityException {
    
    if (str == null || str.length() < 2 || str.charAt(0) != quote || str.charAt(str.length()-1) != quote) {
      throw new InvalidEntityException("INVALID_EXPRESSION_ENTITY",new String[]{str},null);
    }
    
    String result = str.substring(1, str.length()-1);
    
    if (result.indexOf(quote) > -1) {
      StringBuffer buf = new StringBuffer(result.length());
      int i = 0;
      int lengthMinusOne = result.length() - 1;
      while ( i < result.length() ) {
        char c = result.charAt(i);
        buf.append(c);
        if (c == quote) {
          if (i == lengthMinusOne || result.charAt(i+1) != quote) {
            throw new InvalidEntityException("INVALID_STRING_CONSTANT");
          }
          i = i + 1;
        }
        i++;
      }
      return buf.toString();
    }
    
    return result;
  }
  
  
  public static String doubleTheQuotes(String source) {
    
    if (source == null) {
      return null;
    }
    
    int i = source.indexOf('\'');
    if (i == -1) {
      return source;
    }
    StringBuffer buffer = new StringBuffer( source.length() + 10 );
    buffer.append(source.substring(0,i));
    
    for(; i < source.length(); i++) {
      char c = source.charAt(i);
      
      if (c == '\'') {
        buffer.append("\'\'");
      } else {
        buffer.append(c);
      }
    }
    return buffer.toString();
    
  }
  
  public static String toStr(double d) {
    if ((long)d == d) return ((long)d) + "";
    return d + "";
  }
  
  public static String toStr(Object o) {
    if (o instanceof Double) {
      Double d = (Double)o;
      if (d.longValue() == d.doubleValue()) {
        return d.longValue() + "";
      }
      return d.toString();
    }
    return o.toString();
  }
  
  public static String getConstant(String str) throws InvalidEntityException {
    
    if (str == null || str.length() == 0) {
      throw new InvalidEntityException("INVALID_CONSTANT");
    }
    
    if (str.equalsIgnoreCase("true") || str.equalsIgnoreCase("false")) {
      return str;
    }
    
    if (str.charAt(0) == '\'') {
      return getValueOfStringConstant(str, '\'');
    } else {
      try {
        Double.parseDouble(str);
        return str;
      } catch (Exception ignore){}
    }
    
    throw new InvalidEntityException("INVALID_CONSTANT");
  }
  
  private static final String specials = "&'><\"\t\n\f\r ";
  
  private static String codeOf(char c) {
    
    switch (c) {
      case '&'  : return "&amp;";
      case '\'' : return "&apos;";
      case '"'  : return "&quot;";
      case '>'  : return "&gt;";
      case '<'  : return "&lt;";
      case '\t'  : return "&#x9;";
      case '\n'  : return "&#xA;";
      case '\f'  : return "&#xC;"; //C
      case '\r'  : return "&#xD;"; //D
      case ' '  : return "&#xA0;";
    }
    
    return "" + c;
  }
  
  private static String codeOf_NoWhiteSpace(char c){
    switch (c){
      case '&'  : return "&amp;";
      case '\'' : return "&apos;";
      case '"'  : return "&quot;";
      case '>'  : return "&gt;";
      case '<'  : return "&lt;";
      case '\t'  : return "\t";
      case '\n'  : return "\n";
      case '\f'  : return "\f"; //C
      case '\r'  : return "\r"; //D
      case ' '  : return " ";
    }
    return ""+c;
  }
  
  /**
   * Escapes the special charactes in the given string to build a valid XML attribute or value.
   * <P><B>Example:</B></P><PRE>
   *  String s = org.moremotion.evaluator.EvalUtils.escape(" A & B --> 5");
   * </PRE>
   * The value of the s will be <PRE>
   *  "A &amp;amp; B --&amp;gt;5"
   * </PRE>
   *
   *
   * @param str The string that contains the characters to escape.
   * @param preserveWhiteSpace <CODE>true</CODE> means the white space characters should not be escaped.
   * @return The converted string
   */
  public static String escapeForXML(String str, boolean preserveWhiteSpace){
    if (preserveWhiteSpace){
      return escapeForXML(str);
    } else {
      if (str == null) return "";
      StringBuffer sb = new StringBuffer();
      char c;
      for (int i=0; i<str.length(); i++){
        c = str.charAt(i);
        if (specials.indexOf(c) > -1){
          if (c == ' ') { //bosluk ise
            if (sb.length() == 0){
              sb.append(' ');
              continue;
            } else {
              if (sb.charAt(sb.length()-1) != ' '){
                sb.append(' ');
                continue;
              } else {
                sb.append(codeOf_NoWhiteSpace(c));
                continue;
              }
            }
          } else { //bosluk degilse
            if (c == '\r'){ //
              if (i < str.length()-1){
                if (str.charAt(i+1) == '\n'){
                  //bosver bu noktayi
                  i++;
                }
                sb.append(codeOf_NoWhiteSpace(c));
                continue;
                //son noktada degilsek
                
              } else {
                //son noktadayiz
                sb.append(codeOf_NoWhiteSpace(c));
                continue;
              }
            }
            
          }
          sb.append(codeOf_NoWhiteSpace(c));
        } else {
          //ozel karakter degil
          sb.append(c);
        }
      }
      
      return sb.toString();
      //return Text.hashNonAnsiCharacters(sb.toString());
    }
  }
  
  private static String escapeForXML(String str){
    if (str == null) return "";
    StringBuffer sb = new StringBuffer();
    char c;
    for (int i=0; i<str.length(); i++){
      c = str.charAt(i);
      if (specials.indexOf(c) > -1){
        if (c == ' ') { //bosluk ise
          if (sb.length() == 0){
            sb.append(' ');
            continue;
          } else {
            if (sb.charAt(sb.length()-1) != ' '){
              sb.append(' ');
              continue;
            } else {
              sb.append(codeOf(c));
              continue;
            }
          }
        } else { //bosluk degilse
          if (c == '\r'){ //
            if (i < str.length()-1){
              if (str.charAt(i+1) == '\n'){
                //bosver bu noktayi
                i++;
              }
              sb.append(codeOf(c));
              continue;
              //son noktada degilsek
              
            } else {
              //son noktadayiz
              sb.append(codeOf(c));
              continue;
            }
          }
          
        }
        sb.append(codeOf(c));
      } else {
        //ozel karakter degil
        sb.append(c);
      }
    }
    
    return sb.toString();
  }
  
  
  
  public static void main(String[] args) {
    
    System.out.println(  "" + looksLikeSymbol("f:ID/SD") );
    
  }
  
}