/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Funcs.java
 *
 * Created on 08 February 2006 Wednesday, 21:30
 */

package org.moremotion.evaluator;

import java.io.IOException;
import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.Locale;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigManager;

import org.moremotion.config.ConfigurationException;
import org.moremotion.datasrc.DataSourceConfig;
import org.moremotion.datasrc.DataSourceManager;
import org.moremotion.datasrc.Paging;
import org.moremotion.process.ProcessRecord;
import org.moremotion.resource.ResourceException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.adom.ADOMUtils;
import org.moremotion.servlet.SystemConfig;
import org.moremotion.servlet.SystemContext;
import org.moremotion.util.StringParser;
import org.moremotion.util.XString;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Funcs.java 171 2008-07-10 07:46:02Z mustafa $
 */

public class Funcs {
  
  private static final String __uisign = "\5";
  
  public static Object vof(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"vof()"},null);
    }
    
    return params[0].eval(sr);
    
  }
  
  public static Object vov(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"vov()"},null);
    }
    
    String sym = params[0].eval(sr);
    return sym.length() == 0 ? "" : sr.getValue(sym);
    
  }
  
  public static Object resolve(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"resolve()"},null);
    }
    
    String str = params[0].eval(sr);
    
    MScriptResolver resolver = new MScriptResolver(str,(MMSymbolResolver)sr);
    return resolver.resolve();
    
  }
  
  public static Object vore(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 4) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"vore()"},null);
    }
    
    String val = params[0].eval(sr);
    String re  = params[1].eval(sr);
    String sel = params[2].eval(sr);
    String def = params[3].eval(sr);
    
    Pattern p = Pattern.compile(re);
    Matcher m = p.matcher(val);

    if (m.matches()) {
      for (int i = 1; i <= m.groupCount(); i++) {
        String s = "${" + i + "}";
        String v = m.group(i);
        sel = XString.replace(sel,s,v);
      }
      return sel;
    } else {
      return def; 
    }
    
  }
  
  public static Object cdata(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"cdata()"},null);
    }
    
    String value = params[0].eval(sr);
    return "<![CDATA[" + value + "]]>";
    
  }
  
  public static Object not(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"not()"},null);
    }
    
    return new Boolean(!params[0].evalBool(sr));
    
  }
  
  
  public static Object sysparam(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"sysparam()"},null);
    }
    
    String sym = params[0].toString();
    
    String defValue = params.length < 2 || params[1] == null ? "" : params[1].eval(sr);  
    String value = SystemConfig.getParameter(sym,defValue);
    
    return value;
  }
  
  public static Object cfgparam(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"cfgparam()"},null);
    }
    
    String paramName = params[0].eval(sr);
    
    String defValue = params.length < 2 || params[1] == null ? "" : params[1].eval(sr);  
    String value = defValue;
    
    String[] names = XString.split(paramName,':');
    if (names.length != 3) {
      throw new MScriptException("MSCRIPT_ERROR_INVALID_CONFIG_PARAMETER",new String[]{paramName},null);
    }
    
    MoreMotionRequest request = ((MMSymbolResolver)sr).getRequest();
    try {
      Config config = ConfigManager.getConfig(names[0],names[1],request.getOriginPage());
      if (config != null) {
        value = config.getParameter(names[2]).resolve(request).stringValue(defValue);  
      }
    } catch (ConfigurationException ex) {
      throw new MScriptException(ex);
    }  
    
    return value;
  }
  

  public static Object sstr(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"sstr()"},null);
    }
    
    String s = params[0].eval(sr);
    Double d = (Double)(params[1].evaluate(sr));
    int start = d.intValue() - 1;
    if (start >= s.length()) return new String("");
    if (params.length < 3) {
      return new String(s.substring(start));
    }
    d = (Double)(params[2].evaluate(sr));
    int end = d.intValue() + start;
    if (end <= start || end >= s.length()) {
      return new String(s.substring(start));
    }
    return new String(s.substring(start,end));
  }
  
  
  public static Object dspaging(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException, ConfigurationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"dspaging()"},null);
    }
    
    String dsname = params[0].toString();
    if (!EvalUtils.isADOMName(dsname)) {
      throw new MScriptException("MSCRIPT_ERROR_PARAMETER_IS_NOT_ADOM_NAME",new String[]{"dspaging()",dsname},null);
    }
    
    MoreMotionRequest request = ((MMSymbolResolver)sr).getRequest();
    DataSourceConfig dsconfig = DataSourceManager.getDataSourceConfig(request, dsname);
    
    if (dsconfig == null) {
      throw new MScriptException("MSCRIPT_ERROR_DATASOURCE_IS_NULL_OR_INVALID",new String[]{"dspaging()"},null);
    }
    
    
    Paging paging = new Paging( request, dsname, dsconfig);
    
    try {
      if (params[1] != null) {
        Double Dval = (Double)(params[1].evaluate(sr));
        paging.setCurrentPage( Dval.intValue() );
      }
      if (params.length > 2 && params[2] != null) {
        Double Dval = (Double)(params[2].evaluate(sr));
        paging.setItemsPerPage( Dval.intValue() );
      }
      if (params.length > 3 && params[3] != null) {
        Double Dval = (Double)(params[3].evaluate(sr));
        paging.setPagesPerBlock( Dval.intValue() );
      }
    } catch (Exception e) {
      throw new MScriptException("MSCRIPT_ERROR_INVALID_VALUE_FOR_PAGING",new String[]{"dspaging()"},null);
    }
    
    return __uisign;
  }
  
  
  /**
   * @deprecated
   */
  public static Object setnextpage(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"setnp()"},null);
    }
    
    ((MMSymbolResolver)sr).setValue("v:_nextpage", params[0].eval(sr)); 
    return __uisign;
  }
  
  /**
   * @deprecated 
   */
  public static Object getnextpage(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    String s = ((MMSymbolResolver)sr).getValue("v:_nextpage");
    return new String(s);
  }
  
  
  public static Object rsstr(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"rsstr()"},null);
    }
    
    String s = params[0].eval(sr);
    Double d = (Double)(params[1].evaluate(sr));
    int end = s.length() - d.intValue() + 1;
    if (end < 1) return new String("");
    if (params.length < 3) { 
      return new String(s.substring(0,end));
    }
    d = (Double)(params[2].evaluate(sr));
    int start = end - d.intValue();
    if (start < 0) {
      start = 0;
    }
    return new String(s.substring(start,end));
  }
  
  
  public static Object sstra(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"sstra()"},null);
    }
    
    String s = params[0].eval(sr);
    String a = params[1].eval(sr);
    String d = params.length > 2 ? params[2].eval(sr) : "";
    
    return XString.after(s,a,d);
    
  }
  
  public static Object sstral(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"sstral()"},null);
    }
    
    String s = params[0].eval(sr);
    String a = params[1].eval(sr);
    String d = params.length > 2 ? params[2].eval(sr) : "";
    
    return XString.afterLast(s,a,d);
    
  }
  
  public static Object sstrb(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"sstrb()"},null);
    }
    
    String s = params[0].eval(sr);
    String b = params[1].eval(sr);
    String d = params.length > 2 ? params[2].eval(sr) : "";
    
    return XString.before(s,b,d);
    
  }
  
  public static Object sstrbl(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"sstrbl()"},null);
    }
    
    String s = params[0].eval(sr);
    String b = params[1].eval(sr);
    String d = params.length > 2 ? params[2].eval(sr) : "";
    
    return XString.beforeLast(s,b,d);
    
  }
  
  public static Object msfor(Expression[] params, SymbolResolver sr) throws MScriptException {
    
    if (params.length < 4) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"for()"},null);
    }
    
    return params;
    
  }
  
  public static Object chr(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"chr()"},null);
    }
    
    double d = params[0].evalNumber(sr);
    
    return (char)d + "";
    
  }
  
  public static Object chrcode(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"chrcode()"},null);
    }
    
    String s = params[0].eval(sr);
    
    if (s.length() != 1) {
      throw new MScriptException("MSCRIPT_ERROR_SINGLE_CHAR_EXPECTED",new String[]{s},null);
    }
    
    return new Integer((byte)s.charAt(0));
    
  }
  
  public static Object nl(Expression[] params, SymbolResolver sr) {
    
    return "\n";
    
  }
  
  public static Object len(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"len()"},null);
    }
    
    String s = params[0].eval(sr);
    
    return new Double( s.length() );
    
  }
  
  public static Object pos(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"pos()"},null);
    }
    
    String ss = params[0].eval(sr);
    String s = params[1].eval(sr);
    int start = 0;
    if (params.length > 2) {
      Double d = (Double)params[2].evaluate(sr);
      start = d.intValue() - 1;
    }
    
    return new Double( s.indexOf(ss,start) + 1 );
    
  }
  
  public static Object lpos(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"lpos()"},null);
    }
    
    String ss = params[0].eval(sr);
    String s = params[1].eval(sr);
    int start = s.length();
    if (params.length > 2) {
      Double d = (Double)params[2].evaluate(sr);
      start = d.intValue() - 1;
    }
    
    return new Double( s.lastIndexOf(ss,start) + 1 );
    
  }
  
  public static Object trim(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"trim()"},null);
    }
    
    String s = params[0].eval(sr).trim();
    
    return s;
    
  }
  
  public static Object triml(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"triml()"},null);
    }
    
    String s = params[0].eval(sr);
    for (int i = 0; i < s.length(); i++) {
      char c = s.charAt(i);
      if (c != ' ') {
        if (i == 0) return s;
        return s.substring(i);
      }
    }
    return "";
    
  }
  
  public static Object trimr(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"triml()"},null);
    }
    
    String s = params[0].eval(sr);
    for (int i = s.length()-1; i >= 0; i--) {
      char c = s.charAt(i);
      if (c != ' ') {
        if (i == s.length()-1) return s;
        return s.substring(0,i+1);
      }
    }
    return "";
    
  }
  
  
  public static Object isnull(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"isnull()"},null);
    }
    
    String s = params[0].toString();
    
    if (!EvalUtils.isSymbol(s)) {
      throw new MScriptException("MSCRIPT_ERROR_PARAMETER_IS_NOT_SYMBOL",new String[]{"isnull()",s},null);
    }
    return new Boolean(((MMSymbolResolver)sr).isNull(s));
    
  }
  
  public static Object isnum(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"isnum()"},null);
    }
    
    String s = params[0].eval(sr);
    
//    if (s.length() == 0 || s.equals("false") || s.equals("False") || s.equals("true") || s.equals("True")) {
//      return new Boolean(true);
//    }
//    
    return new Boolean(EvalUtils.isNumber(s));
    
  }
  
  public static Object num(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"num()"},null);
    }
    
    Object o = params[0].evaluate(sr);
    if (o instanceof Double) return o;
    
    String s = o.toString().trim();
    if (s.length() == 0 || s.equals("false") || s.equals("False")) return new Double(0);
    if (s.equals("true") || s.equals("True")) return new Double(1);
    try {
      return new Double(s);
    } catch (Exception e) {
      if (params.length > 1) {
        double defvalue = params[1].evalNumber(sr);
        return new Double(defvalue);
      } else {
        throw new MScriptException("MSCRIPT_ERROR_INVALID_NUMBER_ENCOUNTERED",new String[]{"num()",s},null);
      }
    }
    
  }
  
  private static double _round(double val, int places) {
    
    long factor = (long)Math.pow(10,places);
    val = val * factor;
    long tmp = Math.round(val);
    return (double)tmp / factor;
  }
  
  public static Object round(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"round()"},null);
    }
    
    double val = params[0].evalNumber(sr);
    int places = (int)params[1].evalNumber(sr);
    
    return new Double( _round(val, places) );
    
  }
  
  public static Object bool(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"bool()"},null);
    }
    
    String s = params[0].eval(sr);
    boolean isbool = EvalUtils.isBoolean(s);
    if (isbool) {
      return new Boolean(s);
    } else if (EvalUtils.isNumber(s)) {
      double num = EvalUtils.getNumber(s);
      return new Boolean(num > 0);
    } else if (params.length > 1) {
      boolean defvalue = params[1].evalBool(sr);
      return new Boolean(defvalue);
    } else {
      throw new MScriptException("MSCRIPT_ERROR_INVALID_BOOELAN_ENCOUNTERED",new String[]{"bool()",s},null);
    }
  }
  
  
  public static Object encr(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"enc()"},null);
    }
    
    String s = params[0].eval(sr);
    String r = params.length > 1 ? params[1].eval(sr) : "x";
    String key = r + SystemConfig.requestParametersEncryptionKey(); 
    
    return new String(SystemContext.encryptor().encrypt(s,key)); 
    
  }
  
  public static Object decr(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"dec()"},null);
    }
    
    String s = params[0].eval(sr);
    
    String r = params.length > 1 ? params[1].eval(sr) : "x";
    try {
      String key = r + SystemConfig.requestParametersEncryptionKey(); 
      return new String(SystemContext.encryptor().decrypt(s,key));
    } catch (Exception e) {
      throw new MScriptException(e);
    }
    
  }
  
  
  public static Object exists(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"exists()"},null);
    }
    
    String s = params[0].toString();
    
    if (!EvalUtils.isSymbol(s)) {
      throw new MScriptException("MSCRIPT_ERROR_PARAMETER_IS_NOT_SYMBOL",new String[]{"exists()",s},null);
    }
    return new Boolean(!((MMSymbolResolver)sr).isNull(s));
    
  }
  
  
  public static Object shellexec(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException, IOException, InterruptedException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"shellexec()"},null);
    }
    
    String cmd = params[0].eval(sr).trim();
    //    Expression[] parms = (Expression[])params[1].evaluate(sr);
    Array array = (Array)params[1].evaluate(sr);
    Expression[] parms = array.getItems();
    boolean wait = params[2].evalBool(sr);
    
    String[] cmdarr = new String[1 + parms.length];
    cmdarr[0] = cmd;
    
    for (int i = 0; i < parms.length; i++ ) {
      cmdarr[i+1] = parms[i].eval(sr).trim();
    }
    
    Process pro = Runtime.getRuntime().exec(cmdarr);
    if (wait) pro.waitFor();
    
    return new Double(pro.exitValue());
    
  }
  
  
  public static Object ifte(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    Boolean b = (Boolean)params[0].evaluate(sr);
    if (b.booleanValue()) {
      return new String(params[1].eval(sr));
    } else {
      if (params.length > 2) {
        return new String(params[2].eval(sr));
      } else {
        return new String("");
      }
    }
    
  }
  
  public static Object acc(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"acc()"},null);
    }
    
    String sym = params[0].toString();
    if (!EvalUtils.isSymbol(sym)) {
      throw new MScriptException("MSCRIPT_ERROR_PARAMETER_IS_NOT_SYMBOL",new String[]{"acc()",sym},null);
    }
    
    String accby = params[1].eval(sr);
    String dlm = null;
    if (params.length > 2) {
      dlm = params[2].eval(sr);
    }
    
    String currValue = sr.getValue(sym);
    
    String newValue = "";
    if (dlm == null && (currValue.length() == 0 || EvalUtils.isNumber(currValue)) && EvalUtils.isNumber(accby) ) {
      double result = EvalUtils.getNumber(currValue) + EvalUtils.getNumber(accby);
      newValue = EvalUtils.toStr(result);
    } else {
      if (dlm == null) {
        newValue = currValue + accby;
      } else {
        if (currValue.length() == 0) {
          newValue = accby;
        } else {
          newValue = currValue + dlm + accby;
        }
      }
      
    }
    
    sr.setValue(sym, newValue);
    
    return __uisign;
    
  }
  
  public static Object set(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"set()"},null);
    }
    
    String sym = params[0].toString();
    if (!EvalUtils.isSymbol(sym)) {
      throw new MScriptException("MSCRIPT_ERROR_PARAMETER_IS_NOT_SYMBOL",new String[]{"set()",sym},null);
    }
    
    String val = EvalUtils.toStr(params[1].evaluate(sr));
    
    sr.setValue(sym, val);
    
    return __uisign;
    
  }
  
  public static Object push(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"push()"},null);
    }
    
    String sym = params[0].toString();
    if (!EvalUtils.isSymbol(sym)) {
      throw new MScriptException("MSCRIPT_ERROR_PARAMETER_IS_NOT_SYMBOL",new String[]{"push()",sym},null);
    }
    
    Object o = params[1].evaluate(sr);
    
    Stack stack = (Stack)((MMSymbolResolver)sr).getObject(sym);
    if (stack == null) {
      stack = new Stack();
      ((MMSymbolResolver)sr).putObject(sym,stack);
    }
    
    stack.push(o);
    
    return __uisign;
  }
  
  public static Object peek(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"peek()"},null);
    }
    
    String sym = params[0].toString();
    if (!EvalUtils.isSymbol(sym)) {
      throw new MScriptException("MSCRIPT_ERROR_PARAMETER_IS_NOT_SYMBOL",new String[]{"peek()",sym},null);
    }
    
    Stack stack = (Stack)((MMSymbolResolver)sr).getObject(sym);
    if (stack == null || stack.isEmpty()) {
      return "";
    }
    
    return stack.peek();

  }
  
  public static Object pop(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"pop()"},null);
    }
    
    String sym = params[0].toString();
    if (!EvalUtils.isSymbol(sym)) {
      throw new MScriptException("MSCRIPT_ERROR_PARAMETER_IS_NOT_SYMBOL",new String[]{"pop()",sym},null);
    }
    
    Stack stack = (Stack)((MMSymbolResolver)sr).getObject(sym);
    if (stack == null || stack.isEmpty()) {
      return "";
    }
    
    return stack.pop();

  }
  
  public static Object fmtdate(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"fmtdate()"},null);
    }
    String dteStr = params[0].eval(sr);
    Double dbl = new Double(dteStr);
    long dte = dbl.longValue();
    String fmt = params[1].eval(sr);
    
    java.text.SimpleDateFormat format = new java.text.SimpleDateFormat(fmt);
    java.util.Date d = new java.util.Date(dte);
    
    return format.format(d);
  }
  
  public static Object now(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    Calendar cal = java.util.GregorianCalendar.getInstance();
    return new Double(cal.getTime().getTime());
  }
  
  
  public static Object incrdate(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 3) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"incrdate()"},null);
    }
    String dteStr = params[0].eval(sr);
    Double dbl = new Double(dteStr);
    long dte = dbl.longValue();
    
    String typ = params[1].eval(sr);
    
    String amtStr = params[2].eval(sr);
    dbl = new Double(amtStr);
    int amt = dbl.intValue();
    
    GregorianCalendar cal = new GregorianCalendar();
    cal.setTimeInMillis(dte);
    if (typ.equals("YEAR")) cal.add(Calendar.YEAR,amt);
    else if (typ.equals("MONTH")) cal.add(Calendar.MONTH,amt);
    else if (typ.equals("DAY")) cal.add(Calendar.DATE,amt);
    else if (typ.equals("HOUR")) cal.add(Calendar.HOUR,amt);
    else if (typ.equals("MINUTE")) cal.add(Calendar.MINUTE,amt);
    else if (typ.equals("SECOND")) cal.add(Calendar.SECOND,amt);
    else if (typ.equals("MILLISECOND")) cal.add(Calendar.MILLISECOND,amt);
    
    return new Double(cal.getTimeInMillis());
  }
  
  public static Object parsedate(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException, ParseException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"parsedate()"},null);
    }
    String value = params[0].eval(sr);
    String fmt = params[1].eval(sr);
    
    SimpleDateFormat f = new SimpleDateFormat(fmt);
    Date d = f.parse(value);
    return new Double(d.getTime()); 
    
  }
  
  public static Object fmtnum(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"fmtnum()"},null);
    }
    String numStr = params[0].eval(sr);
    Double d = new Double(numStr);
    
    String fmt = params[1].eval(sr);
    DecimalFormatSymbols dfs;
    String lang = null;
    if (params.length > 2) {
      lang = params[2].eval(sr);
    } else {
      lang = ((MMSymbolResolver)sr).getRequest().getLanguage();
    }
    dfs = new DecimalFormatSymbols( new Locale(lang) );
    DecimalFormat f = new DecimalFormat(fmt, dfs);
    
    return f.format(d);
  }
  
  public static Object parsenum(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException, ParseException {
    
    if (params.length < 2) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"parsenum()"},null);
    }
    
    String value = params[0].eval(sr);
    String fmt = params[1].eval(sr);
    
    DecimalFormatSymbols dfs;
    String lang = null;
    if (params.length > 2) {
      lang = params[2].eval(sr);
    } else {
      lang = ((MMSymbolResolver)sr).getRequest().getLanguage();
    }
    dfs = new DecimalFormatSymbols( new Locale(lang) );
    DecimalFormat f = new DecimalFormat(fmt, dfs);
    
    Number n = f.parse(value);
    
    return new Double(n.doubleValue());
    
  }
  
   
  public static Object echo(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"echo()"},null);
    }
    
    String s = params[0].eval(sr);
    System.out.println(s);
    return __uisign;
    
  }
  
   
  public static Object ismodified(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"ismodified()"},null);
    }
    
    String s = params[0].toString();
    
    ProcessRecord prec = ((MMSymbolResolver)sr).peekProcessRecord();
    if (prec != null) {
      String fields = prec.getField("_modified_fields",null);
      if (fields != null) {
        StringParser parser = new StringParser(fields,' ');
        while (parser.hasMore()) {
          if (parser.nextItem().equals(s)) {
            return new Boolean(true);
          }
        }
      }
    }
    return new Boolean(false);
    
  }
  
   
  public static Object filepath(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"filepath()"},null);
    }
    
    String s = params[0].eval(sr);
    if (s.trim().length() == 0) return "";
    return ((MMSymbolResolver)sr).getRequest().getRealFile(s).getAbsolutePath();
    
  }
  
  
  public static Object ucase(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"ucase()"},null);
    }
    
    String s = params[0].eval(sr);
    String lang;
    if (params.length > 1) {
      lang = params[1].eval(sr);
    } else {
      lang = ((MMSymbolResolver)sr).getRequest().getLanguage();
    }
    s = s.toUpperCase( new Locale(lang) );
    
    return s;
    
  }
  
  public static Object lcase(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"ucase()"},null);
    }
    
    String s = params[0].eval(sr);
    String lang;
    if (params.length > 1) {
      lang = params[1].eval(sr);
    } else {
      lang = ((MMSymbolResolver)sr).getRequest().getLanguage();
    }
    s = s.toLowerCase( new Locale(lang) );
    
    return s;
    
  }
  
  
  public static Object dq(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"dq()"},null);
    }
    
    String s = params[0].eval(sr);
    s = EvalUtils.doubleTheQuotes(s);
    
    return s;
    
  }
  
  
  
  public static Object escape(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"escape()"},null);
    }
    
    String source = params[0].eval(sr);
    
    if (source == null || source.length() == 0) {
      return source;
    }
    
    boolean quot = true;
    boolean apos = true;
    boolean lt = true;
    boolean gt = true;
    boolean amp = true;
    
    if (params.length > 1) {
      
      quot = false;
      apos = false;
      lt = false;
      gt = false;
      amp = false;
      
      String s = params[1].eval(sr);
      for (int i = 0; i < s.length(); i++) {
        char c = s.charAt(i);
        switch (c) {
          case '&': amp = true; break;
          case '<': lt = true; break;
          case '>': gt = true; break;
          case '"': quot = true; break;
          case '\'': apos = true; break;
        }
      }
    }
    
    int i;
    StringBuffer buffer = null;
    for(i = 0; i < source.length(); i++) {
      char c = source.charAt(i);
      if ( (c == '&' && amp) || (c == '<' && lt) || (c == '>' && gt) || (c == '"' && quot) || (c == '\'' && apos) ) {
        buffer = new StringBuffer( source.length() + 10 );
        buffer.append(source.substring(0,i));
        break;
      }
    }
    
    if (buffer == null) return source;
    
    for(; i < source.length(); i++) {
      
      char c = source.charAt(i);
      
      switch (c) {
        
        case '&':  if (amp) { buffer.append("&amp;");  break; }
        case '\'': if (apos) { buffer.append("&apos;"); break; }
        case '>': if (gt) { buffer.append("&gt;"); break; }
        case '<': if (lt) { buffer.append("&lt;"); break; }
        case '"': if (quot) { buffer.append("&quot;"); break; }
        
        default:
          buffer.append(c);
      }
    }
    
    return buffer.toString();
    
  }
  
  public static Object repl(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 3) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"repl()"},null);
    }
    
    String source = params[0].eval(sr);
    
    if (source == null || source.length() == 0) {
      return source;
    }
    
    Object o1 = params[1].evaluate(sr);
    Object o2 = params[2].evaluate(sr);
    
    if ((o1 instanceof Array) || (o2 instanceof Array));
    else {
      String f = params[1].eval(sr);
      String t = params[2].eval(sr);
      return XString.replace(source,f,t);
    }
    
    Expression[] fr = ((Array)o1).getItems();
    Expression[] to = ((Array)o2).getItems();
    
    int frl = fr.length;
    
    if (frl != to.length) {
      throw new MScriptException("MSCRIPT_ERROR_NUMBER_OF_ARRAY_ELEMENTS_INCONSISTENT",
      new String[]{"repl()"},null);
    }
    
    if (frl == 1) {
      String f = fr[0].eval(sr);
      String t = to[0].eval(sr);
      return XString.replace(source,f,t);
    }
    
    String[] frs = new String[fr.length];
    String[] tos = new String[to.length];
    for (int i = 0; i < frl; i++) {
      frs[i] = fr[i].eval(sr);
      tos[i] = to[i].eval(sr);
    }
    
    char[] frc = new char[frl];
    for (int i = 0; i < frl; i++) {
      if (frs[i].length() == 1) {
        frc[i] = frs[i].charAt(0);
      } else {
        frc = null;
        break;
      }
    }
    
    StringBuffer buffer = new StringBuffer(source.length() * 2);
    
    if (frc != null) {
      for (int i = 0; i < source.length(); i++) {
        char c = source.charAt(i);
        
        boolean done = false;
        for (int j = 0; j < frl; j++) {
          if (c == frc[j]) {
            buffer.append(tos[j]);
            done = true;
            break;
          }
        }
        if (!done) {
          buffer.append(c);
        }
      }
    } else {
      
      int sl = source.length();
      int i = 0;
      for (; i < sl; i++) {
        
        boolean found = false;
        
        for (int j = 0; j < frl; j++) {
          String s = frs[j];
          int le = s.length();
          int k = i;
          int l = 0;
          while (k < sl && l < le) {
            if (source.charAt(k) != s.charAt(l)) {
              break;
            }
            k++;
            l++;
          }
          found = l == le;
          if (found) {
            buffer.append(tos[j]);
            i = k - 1;
            break;
          }
        }
        
        if (!found) {
          buffer.append(source.charAt(i));
        }
      }
    }
    
    return buffer.toString();
    
  }
  
  public static Object res(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException, ResourceException {
    
    if (params.length < 3) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"res()"},null);
    }
    
    String msg = params[0].eval(sr);
    
    Expression[] prms = null;
    if (params[1] != null) {
      Array a = (Array)params[1].evaluate(sr);
      prms = a.getItems();
    }
    String unit = params[2].eval(sr);
    
    String[] resprms = null;
    if (prms != null && prms.length > 0) {
      resprms = new String[prms.length];
      for (int i = 0; i < prms.length; i++) {
        resprms[i] = prms[i].eval(sr);
      }
    }
    
    String lang;
    if (params.length > 3) {
      lang = params[3].eval(sr);
    } else {
      lang = ((MMSymbolResolver)sr).getRequest().getLanguage();
    }
    org.moremotion.resource.Resource res = new org.moremotion.resource.Resource(msg,resprms,unit);
    
    return res.format(lang);
    
  }
  
  public static Object msg(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 1) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"msg()"},null);
    }
    
    String msg = params[0].eval(sr);
    
    String[] resprms = null;
    if (params.length > 1) {
      Array a = (Array)params[1].evaluate(sr);
      Expression[] prms = a.getItems();
      if (prms != null && prms.length > 0) {
        resprms = new String[prms.length];
        for (int i = 0; i < prms.length; i++) {
          resprms[i] = prms[i].eval(sr);
        }
      }
    }
    
    String dets = "";
    if (params.length > 2) {
      dets = params[2].eval(sr);
    }
    
    String unit = "";
    if (params.length > 3) {
      unit = params[3].eval(sr);
    }
    
    MMSymbolResolver mmsr = (MMSymbolResolver)sr;
    MoreMotionRequest request = mmsr.getRequest();
    request.createMessage(unit, msg, resprms, dets);
    
    return __uisign;
    
  }
  
  
  
  // accfa(s:cart,PRICE > 100,'ID = ' + ID, ' OR ')
  public static Object accfa(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 3) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"accfa()"},null);
    }
    
    String adomName = params[0].toString();
    if (!EvalUtils.isADOMName(adomName)) {
      throw new MScriptException("MSCRIPT_ERROR_PARAMETER_IS_NOT_ADOM",new String[]{"accfa()",adomName},null);
    }
    
    Expression precond = params[1] == null ? null : params[1];
    
    MMSymbolResolver mmsr = (MMSymbolResolver)sr;
    
    MoreMotionRequest request = mmsr.getRequest();
    ADOM adom = ADOMUtils.findADOM(request, adomName);
    if (adom == null) {
      if (EvalUtils.isNumber(params[2].toString())) return new Double(0);
      return "";
    }
    
    
    Expression dlm = params.length > 3 ? params[3] : null;
    
    String accstr = "";
    
    ADOMNode savedNode = mmsr.getADOMNode();
    
    Iterator it = adom.getNodes();
    while (it.hasNext()) {
      
      ADOMNode node = (ADOMNode)it.next();
      
      if (node.getName().equals("item")) {
        mmsr.setADOMNode(node);
        
        boolean satisfied = precond == null || precond.evalBool(mmsr);
        
        if (satisfied) {
          
          String accbyValue = params[2].eval(mmsr);
          
          String dlmValue = dlm == null ? null : dlm.eval(mmsr);
          if (dlmValue == null && (accstr.length() == 0 || EvalUtils.isNumber(accstr)) && EvalUtils.isNumber(accbyValue) ) {
            double result = EvalUtils.getNumber(accstr) + EvalUtils.getNumber(accbyValue);
            accstr = EvalUtils.toStr(result);
          } else {
            if (dlmValue == null) {
              accstr = accstr + accbyValue;
            } else {
              if (accstr.length() == 0) {
                accstr = accbyValue;
              } else {
                accstr = accstr + dlmValue + accbyValue;
              }
            }
          }
        }
      }
    }
    
    mmsr.setADOMNode(savedNode);
    
    if (accstr.length() > 0 && EvalUtils.isNumber(accstr)) return new Double(accstr);
    if (accstr.length() == 0 && EvalUtils.isNumber(params[2].toString())) return new Double(0);
    return accstr;
  }
  
  // accfs('A,B;C',',',? > 100,'ID = ' + ID, ' OR ')
  public static Object accfs(Expression[] params, SymbolResolver sr) throws MScriptException, EvaluationException {
    
    if (params.length < 4) {
      throw new MScriptException("MSCRIPT_ERROR_LESS_PARAMETERS_THAN_EXPECTED",new String[]{"accfs()"},null);
    }
    
    String str = params[0].eval(sr);
    String spr = params[1] == null ? "" : params[1].eval(sr);
    Expression precond = params[2] == null ? null : params[2];
    Expression acc = params[3];
    Expression dlm = params.length > 4 ? params[4] : null;
    
    MMSymbolResolver mmsr = (MMSymbolResolver)sr;
    String[] tokens = null;
    
    if (spr.length() == 0) {
      tokens = new String[str.length()];
      for (int i = 0; i < str.length(); i++) {
        tokens[i] = str.charAt(i) + "";
      }
      
    } else {
      ArrayList a = new ArrayList();
      StringParser p = new StringParser(str,spr.charAt(0));
      p.trimmed();
      while (p.hasMore()) {
        a.add( p.nextItem() );
      }
      tokens = new String[a.size()];
      a.toArray(tokens);
    }
    
    String accstr = "";
    
    for (int i = 0; i < tokens.length; i++) {
      sr.setValue(".", tokens[i]);
      
      boolean satisfied = precond == null || precond.evalBool(sr);
      
      if (satisfied) {
        
        String accbyValue = acc.eval(mmsr);
        
        String dlmValue = dlm == null ? null : dlm.eval(mmsr);
        if (dlmValue == null && (accstr.length() == 0 || EvalUtils.isNumber(accstr)) && EvalUtils.isNumber(accbyValue) ) {
          double result = EvalUtils.getNumber(accstr) + EvalUtils.getNumber(accbyValue);
          accstr = EvalUtils.toStr(result);
        } else {
          if (dlmValue == null) {
            accstr = accstr + accbyValue;
          } else {
            if (accstr.length() == 0) {
              accstr = accbyValue;
            } else {
              accstr = accstr + dlmValue + accbyValue;
            }
          }
        }
      }
    }
    
    
    if (accstr.length() > 0 && EvalUtils.isNumber(accstr)) return new Double(accstr);
    if (accstr.length() == 0 && EvalUtils.isNumber(params[2].toString())) return new Double(0);
    return accstr;
  }
  
  
  public static void main(String[] args) {
    
    try {
      SimpleDateFormat f = new SimpleDateFormat("yyyy-MM-dd hh:mm");
      Date d = f.parse("2000-11-23 12:24");
      GregorianCalendar cal = new GregorianCalendar();
      cal.setTimeInMillis(d.getTime());
      System.out.println("" + cal.getTimeInMillis());
      cal.add(Calendar.MONTH,10);
      System.out.println("" + cal.getTimeInMillis());
      
      System.out.println( f.format(  cal.getTime() ) );
      
    } catch( Exception e) {e.printStackTrace();}
  }
  
}
