/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * MMSymbolResolver.java
 *
 * Created on 27 June 2004 Monday, 22:49
 */

package org.moremotion.evaluator;

import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Stack;
import org.moremotion.adom.ADOM;
import org.moremotion.adom.ADOMNode;
import org.moremotion.cache.VariablePool;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.page.UserAgentAnalyzer;
import org.moremotion.process.ProcessRecord;
import org.moremotion.process.CallInstance;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.adom.ADOMUtils;
import org.moremotion.servlet.SystemConfig;
import org.moremotion.util.Logger;
import org.moremotion.util.MiscUtils;
import org.moremotion.util.XString;
import org.moremotion.servlet.SystemConfig;


/**
 * The Global Symbol Resolver for all types of symbols used in a MoreMotion environment:
 * <UL>
 * <LI>Input Parameters, i.e. <CODE>i:NAME</CODE></LI>
 * <LI>Process Record Fields, i.e. <CODE>f:COUNT</CODE></LI>
 * <LI>Pool Variables, i.e. <CODE>v:_maxcc</CODE></LI>
 * <LI>Datasource Node symbols, i.e. <CODE>s:/mor_security_uinfo_1/username</CODE></LI>
 * </UL>
 * @author Mehmet Tastan 
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: MMSymbolResolver.java 162 2008-06-30 16:03:12Z erkan $
 */
public class MMSymbolResolver implements SymbolResolver {
  
  public static final int ADOM_NODE_PROCESSING_MODE_IGNORE = 0;
  public static final int ADOM_NODE_PROCESSING_MODE_FIRST = 1;
  public static final int ADOM_NODE_PROCESSING_MODE_LAST = 2;
  
  private MoreMotionRequest request = null;
  private ADOMNode adomnode = null;
  private VariablePool vpool = null;
  private boolean doubleBackSlashes = false;
  private int adomNodeProcessingMode = ADOM_NODE_PROCESSING_MODE_FIRST;
  private boolean disableQSpecifier = false;
  private String dotValue = "";
  private Logger log = null;
  private HashMap objects = null;
  
  private ProcessRecord prec;
  private Stack precStack;
  
  private HashMap tcall;
  private Stack tcallStack;
  
  private Stack recurseStack;
  
  private CallInstance processCallInstance = null;
  
  public boolean bypassRIE = false;
  public SymbolResolver assistant = null;
  
  /** Creates a new instance of SymbolResolver */
  public MMSymbolResolver(MoreMotionRequest request)  {
    this.request = request;
    this.vpool = request.getVPool();
    recurseStack = new Stack();
  }
  
  public void setProcessCallInstance(CallInstance ci) {
    this.processCallInstance = ci;
  }
  
  public CallInstance getProcessCallInstance() {
    return this.processCallInstance;
  }
  
  public void setLogger(Logger log) {
    this.log = log;
  }
  
  public Logger getLogger() { 
    return this.log;
  }
  
  public void setDisableQSpecifier(boolean value) {
    this.disableQSpecifier = value;
  }
  
  public boolean getDisableQSpecifier() {
    return this.disableQSpecifier;
  }
  
  public void pushProcessRecord(ProcessRecord prec) {
    if (precStack == null) {
      precStack = new Stack();
    }
    precStack.push(prec);
    this.prec = prec;
  }
  
  public void popProcessRecord() {
    
    if (precStack == null || precStack.isEmpty()) {
      this.prec = null;
      return;
    }
    
    precStack.pop();
    if (precStack.isEmpty()) {
      this.prec = null;
    }
    
  }
  
  public ProcessRecord peekProcessRecord() {
    if (precStack == null || precStack.isEmpty()) {
      return null;
    }
    return (ProcessRecord) precStack.peek();
  }
  
  private HashMap getResolvedParams(HashMap tcall) throws ConfigParameterException {
    
    if (tcall == null) return null;
    
    HashMap copy = new HashMap(tcall.size());
    Iterator itr = tcall.entrySet().iterator();
    while (itr.hasNext()) {
      Map.Entry entry = (Map.Entry)itr.next();
      String key = (String)entry.getKey();
      Object valueObject = entry.getValue();
      if (valueObject instanceof String) {
        copy.put(key,valueObject);
      } else {
        ConfigParameter value = (ConfigParameter)valueObject;
        copy.put(key,value.resolve(request));
      }
    }
    return copy;
  }
  
  public void pushTCall(HashMap tcall) throws ConfigParameterException {
    if (tcallStack == null) {
      tcallStack = new Stack();
    }
    tcallStack.push( getResolvedParams(tcall) );
    this.tcall = tcall;
  }
  
  public void popTCall() {
    
    if (tcallStack == null || tcallStack.isEmpty()) {
      this.tcall = null;
      return;
    }
    
    tcallStack.pop();
    if (tcallStack.isEmpty()) {
      this.tcall = null;
    }
    
  }
  
  public void setADOMNode(ADOMNode adomnode) {
    this.adomnode = adomnode;
  }
  
  public ADOMNode getADOMNode() {
    return this.adomnode;
  }
  
  public void setAdomNodeProcessingMode(int value) {
    this.adomNodeProcessingMode = value;
  }
  
  public void resetAdomNodeProcessingMode() {
    this.adomNodeProcessingMode = ADOM_NODE_PROCESSING_MODE_FIRST;
  }
  
  public MoreMotionRequest getRequest() {
    return this.request;
  }
  
  public VariablePool getVPool() {
    return this.request.getVPool();
  }
  
  public boolean getBoolValue(String symbol) throws EvaluationException {
    return getValue(symbol).equalsIgnoreCase("true") ? true : false;
  }
  
  public double getNumberValue(String symbol) throws EvaluationException {
    double result = 0;
    try {
      result = Double.parseDouble(getValue(symbol));
    } catch (Exception e){}
    return result;
  }
  
  public String getStringValue(String symbol) throws EvaluationException {
    return getValue(symbol);
  }
  
  public int getType(String symbol) throws EvaluationException {
    if (assistant != null) {
      int result = assistant.getType(symbol);
      if (result > DT_NULL) return result;
    }
    if (symbol == null || symbol.trim().equals("")) return DT_STRING;
    if (symbol.equals("_maxcc") && processCallInstance != null) return DT_NUMBER;
    if (symbol.equals("_maxlcc") && processCallInstance != null) return DT_NUMBER;
    if (symbol.equals("_modified")) return DT_BOOL;
    if (symbol.equals("_selected")) return DT_BOOL;
    if (symbol.startsWith("_") && symbol.endsWith("_cc")) return DT_NUMBER;
    
    String scope = XString.before(symbol,':'," ").toLowerCase();
    if (scope.indexOf('n') > -1) return DT_NUMBER;
    else if (scope.indexOf('b') > -1) return DT_BOOL;
    else return DT_STRING;
    
  }
  
  public String getValue(String symbol) throws EvaluationException {
    
    if (assistant != null) {
      String result = assistant.getValue(symbol);
      if (result != null) return result;
    }
    
    if (recurseStack.contains(symbol)) {
      throw new EvaluationException("MSCRIPT_SYMBOL_CIRCULAR_REFERENCE",new String[]{symbol},null);
    }
    recurseStack.push(symbol);
    
    String result = _getValue(symbol);
    
    recurseStack.pop();
    
    if (result == null) {
      result = "";
    }
    
    return result;
  }
  
  public boolean isNull(String symbol) throws SymbolResolvingException {
    
    return _getValue(symbol) == null;
    
  }
  
  private String _getValue(String symbol) throws SymbolResolvingException {
  
    if (symbol.equals(".")) return dotValue;
    
    if (symbol.charAt(0) == '_') {
      
      if (symbol.equals("_maxcc")) {
        if (processCallInstance != null) {
          return "" + processCallInstance.getMaxCompletionCode();
        } else {
          return vpool.get(symbol,"0");
        }
      }
      if (symbol.equals("_maxlcc") && processCallInstance != null) {
        return "" + processCallInstance.getMaxLevelCompletionCode();
      }

      if (symbol.endsWith("_cc")) return vpool.get(symbol,"0");
    
      if (symbol.equals("_nextpage") || symbol.equals("_errorpage") || symbol.equals("_continuepage")) {
        return vpool.get(symbol,"");
      }
      
    }
    
    String result = null;
    
    Symbol sym = new Symbol(symbol);
    String error = sym.getError();
    if (error != null) {
      throw new SymbolResolvingException(error,new String[]{symbol},null);
    }

    char src = sym.getSTS();
    String symn = sym.getName();
        
    if (sym.isAdomName()) {
      result = getADOMValue(symbol);
      
    } else
      
      assign:
      {
        if (src == ' ') {
          
          if (adomnode != null && adomNodeProcessingMode == ADOM_NODE_PROCESSING_MODE_FIRST) {
            result = getItemNodeValue(adomnode,symn);
            if (result != null) break assign;
          }
          
          if (prec != null) {
            result = getProcessFieldValue(sym);
            if (result != null) break assign;
          }
          
          {
            result = vpool.get(symn);
            if (result != null) break assign;
          }
          
          {
            result = getTCallSymbolValue(sym);
            if (result != null) break assign;
          }
          
          if (SystemConfig.iSymbolFirst()) {
            result = request.getParameter(symn);
            if (result != null) break assign;
          }
          
          {
            result = request.getVars().getPageVar(symn);
            if (result != null) break assign;
          }
          
          {
            result = request.getVars().getSessionVar(symn);
            if (result != null) break assign;
          }
          
          if (!SystemConfig.iSymbolFirst()) {
            result = request.getParameter(symn);
            if (result != null) break assign;
          }
          
          if (adomnode != null && adomNodeProcessingMode == ADOM_NODE_PROCESSING_MODE_LAST) {
            result = getItemNodeValue(adomnode,symn);
          }
          
        } else if (src == 'v') {
          result = vpool.get(symn);
          
        } else if (src == 'a') {
          
          if (adomnode == null) {
            throw new SymbolResolvingException("MSCRIPT_ERROR_SOURCE_UNAVAILABLE",new String[]{""+src, symbol},null);
          }
          result = getItemNodeValue(adomnode,symn);
          
        } else if (src == 'f') {
          
          if (prec != null) {
            result = getProcessFieldValue(sym);
          } else {
            throw new SymbolResolvingException("MSCRIPT_ERROR_SOURCE_UNAVAILABLE",new String[]{""+src, symbol},null);
          }
           
        } else if (src == 'i') {
          result = request.getParameter(symn);
          
        } else if (src == 'p') {
          result = request.getVars().getPageVar(symn);
          
        } else if (src == 's') {
          result = request.getVars().getSessionVar(symn);
          
        } else if (src == 'c') {
          result = getTCallSymbolValue(sym);
          
        } 
      }
      
      if (!disableQSpecifier && result != null && sym.getSTS() == 'q' && result.indexOf('\'') > -1) {
        return doubleTheQuotes(result);
      }
      
      return result;
      
  }
  
  private void setProcessFieldValue(Symbol sym, String value) {
    
    ProcessRecord _prec = getProcessRecord(sym);
    
    if (_prec != null) {
      _prec.setField(sym.getName(), value);
    }
  }
  
  private ProcessRecord getProcessRecord(Symbol sym) {
    
    if (precStack == null || precStack.isEmpty() ) {
      return null;
    }
    
    ProcessRecord _prec = null;
    if (sym.getDepth() <= 0) {
      _prec = this.prec;
      
    } else {
      
      int idx = precStack.size() - 1 - sym.getDepth();
      if (idx < 0) {
        return null;
      }
      _prec = (ProcessRecord)precStack.get(idx);
    }
    
    return _prec;
  }
  
  private String getProcessFieldValue(Symbol sym) {
    
    if (precStack == null || precStack.isEmpty() ) {
      return null;
    }
    
    ProcessRecord _prec = null;
    
    if (sym.getDepth() == 0) {
      return this.prec.getField(sym.getName(), null);
      
    } else if (sym.getDepth() == -1) {  
      
      int idx = precStack.size() - 1;
      while (idx >= 0) {
        _prec = (ProcessRecord)precStack.get(idx);
        String val = _prec.getField(sym.getName(), null);
        if (val != null) return val;
        idx--;
      }
      return null;
      
    } else {
      
      int idx = precStack.size() - 1 - sym.getDepth();
      if (idx < 0) {
        return null;
      }
      _prec = (ProcessRecord)precStack.get(idx);
      return _prec.getField(sym.getName(),null);
      
    }
    
  }
  
  /*
   * Template Call Symbol Value
   */
  private String getTCallSymbolValue(Symbol sym) throws SymbolResolvingException {
    
    Object o = getTCallParam(sym);
    if (o == null) return null;
    
    if (o instanceof String) {
      return (String)o;
    } else if (o instanceof ConfigParameter) {
      ConfigParameter p = (ConfigParameter)o;
      try {
        return p.resolve(request).stringValue(null);
      } catch (ConfigParameterException ex) {
        throw new SymbolResolvingException(ex);
      }
    }
    return null;
        
  }
   
  private Object getTCallParam(Symbol sym) {
    
    if (tcallStack == null || tcallStack.isEmpty() ) {
      return null;
    }
    
    if (sym.getDepth() == 0) {
      return this.tcall == null ? null : this.tcall.get(sym);
      
    } else if (sym.getDepth() == -1) {  
      
      int idx = tcallStack.size() - 1;
      while (idx >= 0) {
        HashMap map = (HashMap)tcallStack.get(idx);
        if (map != null) {
          Object o = map.get(sym.getName());
          if (o != null) return o;
        }
        idx--;
      }
      return null;
      
    } else {
      
      int idx = tcallStack.size() - 1 - sym.getDepth();
      if (idx < 0) {
        return null;
      }
      HashMap map = (HashMap)tcallStack.get(idx);
      return map == null ? null : map.get(sym);
    }
    
  }
  
  
  
  public String doubleTheQuotes(String source) {
    
    if (source == null) {
      return null;
    }
    
    int i;
    StringBuffer buffer = null;
    for(i = 0; i < source.length(); i++) {
      char c = source.charAt(i);
      if (c == '\'' || c == '\\') {
        buffer = new StringBuffer( source.length() + 10 );
        buffer.append(source.substring(0,i));
        break;
      }
    }
    
    if (buffer == null) return source;
    
    for(; i < source.length(); i++) {
      
      char c = source.charAt(i);
      
      switch (c) {
        case '\\':
          if (doubleBackSlashes) {
            buffer.append("\\\\");
          } else {
            buffer.append(c);
          }
          break;
          
        case '\'':
          buffer.append("\'\'");
          break;
          
        default:
          buffer.append(c);
      }
    }
    return buffer.toString();
    
  }
  
  
  public void setValue(String symbol, String value) throws SymbolResolvingException  {
    
    if (symbol.equals(".")) {
      dotValue = value;
      return;
    }
    
    if (symbol.length() > 0 && symbol.charAt(0) == '_') {
      if ((symbol.equals("_maxcc") || symbol.equals("_maxlcc")) && processCallInstance != null) {
        int maxlcc = XString.create(value).i(0);
        processCallInstance.resetCompletionCode(maxlcc);
        return;
      }
        
      if (symbol.equals("_nextpage") || symbol.equals("_errorpage") || symbol.equals("_continuepage")) {
        vpool.put(symbol, value);
        return;
      }
    }
    
    if (symbol.indexOf('/') > -1) {
      setADOMValue(symbol, value);
    } else {
      
      Symbol sym = new Symbol(symbol);
      String error = sym.getErrorForTargetSymbol();
      if (error != null) {
        throw new SymbolResolvingException(error,new String[]{symbol},null);
      }
      
      char tar = sym.getSTS();
      String symn = sym.getName();
      
      if (tar == ' ') {
        if (adomnode != null) {
          setItemNodeValue(adomnode, symn, value);
        } else if (prec != null) {
          setProcessFieldValue(sym, value);
        } else {
          vpool.put(symn, value);
        }
        
      } else if (tar == 'v') {
        vpool.put(symn, value);
        
      } else if (tar == 'p') {
        request.getVars().setPageVar(symn, value);
        
      } else if (tar == 's') {
        request.getVars().setSessionVar(symn, value);
        
      } else if (tar == 'f') {
        if (prec != null) {
          setProcessFieldValue(sym, value);
        } else {
          throw new SymbolResolvingException("MSCRIPT_ERROR_TARGET_UNAVAILABLE",new String[]{""+tar, symbol},null);
        }
        
      } else if (tar == 'a') {
        if (adomnode != null) {
          setItemNodeValue(adomnode, symn, value);
        } else {
          throw new SymbolResolvingException("MSCRIPT_ERROR_TARGET_UNAVAILABLE",new String[]{""+tar, symbol},null);
        }
        
      } 
      
    }
    
  }
  
  private void setItemNodeValue(ADOMNode item, String sym, String value) {
    if (sym.charAt(0) == '@') {
      item.addAttribute(sym.substring(1), value);
    } else {
      item.setNodeValue(sym, value);
    }
  }
  
  private String getItemNodeValue(ADOMNode item, String sym) {
    if (sym.charAt(0) == '@') {
      return item.getAttribute(sym.substring(1));
    }
    ADOMNode node = item.findNode(sym);
    if (node != null) {
      return node.getValue();
    }
    return null;
  }
  
  private String getADOMValue(String symbol) {
    
    String result = null;
    
    symbol = symbol.replace('^','_').replace('.','/');
    String nodePath = ADOMUtils.getNodePath(symbol);
    if (nodePath == null) return null;

    String adomName = ADOMUtils.getPureADOMName(symbol);
    if (adomName.equals("request")) {
      return getRequestInfo(ADOMUtils.getNodePath(symbol));
    } else if (adomName.equals("servertime")) {
      return getTimeInfo(ADOMUtils.getNodePath(symbol));
    }

    ADOM adom = ADOMUtils.findADOM(request, symbol);
    if (adom != null) {
      ADOMNode node = adom.findNode(nodePath);
      if (node != null) {
        result = node.getValue();
      }
    }
    
    return result;
  }
  
  private String getRequestInfo(String sym) {
    
    if (sym.equals("ApplicationPath")) return request.getApplicationPath();
    else if (sym.equals("SessionId")) return request.getSession().getId();
    else if (sym.equals("RemoteAddr")) return request.getRemoteAddr();
    else if (sym.equals("RemoteHost")) return request.getRemoteHost();
    else if (sym.equals("RemoteUser")) return request.getRemoteUser();
    else if (sym.equals("RequestURI")) return request.getRequestURI();
    else if (sym.equals("BrowserIsXSLTCapable")) return UserAgentAnalyzer.isXSLTCapable(request);
    else if (sym.equals("XSLTInBrowser")) return "" + UserAgentAnalyzer.isXSLTEnabled(request);
    else if (sym.equals("ServletPath")) return request.getServletPath();
    else if (sym.equals("AuthType")) return request.getAuthType();
    else if (sym.equals("CharacterEncoding")) return request.getCharacterEncoding();
    else if (sym.equals("ContentType")) return request.getContentType();
    else if (sym.equals("ContextPath")) return request.getContextPath();
    else if (sym.equals("Method")) return request.getMethod();
    else if (sym.equals("PathInfo")) return request.getPathInfo();
    else if (sym.equals("PathTranslated")) return request.getPathTranslated();
    else if (sym.equals("Protocol")) return request.getProtocol();
    else if (sym.equals("QueryString")) return request.getQueryString();
    else if (sym.equals("RequestedSessionId")) return request.getRequestedSessionId();
    else if (sym.equals("Scheme")) return request.getScheme();
    else if (sym.equals("UserAgent")) return request.getHeader("user-agent");
    else if (sym.equals("Referer")) return request.getHeader("referer");
    else if (sym.equals("LanguageCode")) return request.getLanguage(); 
    else if (sym.equals("applicationLanguage")) return SystemConfig.applicationLanguage(); 
    else return request.getParameter(sym,"");
    
  }
  
  private String getTimeInfo(String sym) {
    Calendar cal = java.util.GregorianCalendar.getInstance();
    int zoneOffset = cal.get(Calendar.ZONE_OFFSET);
    Date dt = cal.getTime();
    if (sym.equals("time")) return "" + dt.getTime();
    else if (sym.equals("timestamp")) return MiscUtils.timeStamp();
    else if (sym.equals("zoneoffset")) return "" + zoneOffset;
    else return "";
  }
  
  
  private void setADOMValue(String symbol, String value) {
    if (symbol == null || symbol.length() == 0) return;
    
    symbol = symbol.replace('^','_').replace('.','/');
    ADOM adom = ADOMUtils.findADOM(request, symbol);
    if (adom == null) {
      char scope = ADOMUtils.getADOMScope(symbol);
      String adomKey = ADOMUtils.getPureADOMName(symbol);
      if (scope == 'r') adom = request.newRequestADOM(adomKey);
      else if (scope == 'a') adom = request.newApplicationADOM(adomKey);
      else adom = request.newSessionADOM(adomKey);
    }
    String nodePath = ADOMUtils.getNodePath(symbol);
    if (nodePath != null) {
      adom.setNodeValue(nodePath, value);
    }
    
  }
  
  public void setDoubleBackSlashes(boolean value) {
    doubleBackSlashes = value;
  }
  
  public boolean doubleTheBackSlashes() {
    return this.doubleBackSlashes;
  }
  
  public void putObject(String name, Object o) {
    if (objects == null) objects = new HashMap();
    objects.put(name, o);
  }
  
  public Object getObject(String name) {
    if (objects == null) return null;
    return objects.get(name);
  }
  
  
  public static void main(String[] args) throws Exception{
  }
  
}


