/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * CommandDefinition.java
 *
 * Created on 12 July 2006 Wednesday, 21:52
 */

package org.moremotion.page;


import java.util.LinkedList;
import org.moremotion.config.ConditionalList;
import org.moremotion.config.ConditionalListItem;
import org.moremotion.config.ConditionalListIterator;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigurationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Element;


/**
 * A CommandDefinition defines a command that can be used to navigate to a certain MoreMotion page.
 * It includes a command name and parameters.
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: CommandDefinition.java 222 2008-09-29 09:37:05Z erkan $
 */
public class CommandDefinition extends ConditionalListItem {
  
  private String name;
  private ConditionalList params = null;
  public ConfigParameter clearPageVars;
  public ConfigParameter clearSessionVars;
  
  protected CommandDefinition(Element elm) throws ConfigurationException {
    
    super(elm);
    
    name = elm.getAttribute("name");
    clearPageVars = new ConfigParameter("clearPageVars",elm.getAttribute("clearPageVars"));
    clearSessionVars = new ConfigParameter("clearSessionVars",elm.getAttribute("clearSessionVars"));
    
    NodeList nlist = elm.getChildNodes();
    for (int i = 0; i < nlist.getLength(); i++) {
      Node node = nlist.item(i);
      if (node.getNodeType() == Node.ELEMENT_NODE) {
        if (params == null) {
          params = new ConditionalList();
        }
        params.add( new CommandParameter( (Element)nlist.item(i) ) );
      }
    }
    
  }
  
  public String getName() {
    return this.name;
  }
  
  public ParmToEncrypt[] getNodesToEncrypt(MoreMotionRequest request) throws Exception {
    
    if (params == null) return new ParmToEncrypt[0];
    
    LinkedList ll = new LinkedList();
    ConditionalListIterator itr = params.iterator(request);
    while (itr.hasMore()) {
      CommandParameter param = (CommandParameter)itr.next();
      if (param.encrypted(request)) {
        ll.addLast( new ParmToEncrypt(param.name(),param.encrkey(request)));
      }
    }
    
    ParmToEncrypt[] result = new ParmToEncrypt[ll.size()];
    ll.toArray(result);
    
    return result;
  }
  
  public ConditionalList getParams() {
    return this.params;
  }
  
  
  protected void checkRequestParameters(MoreMotionRequest request) throws PageGenerationException {
    
    try {
      
      String value = null;
      
      value = clearPageVars.resolve(request).stringValue("");
      if (value.equals("*") || value.equalsIgnoreCase("true") || value.equalsIgnoreCase("all")) {
        request.getVars().removeAllPageVars();
      } else if (value.equalsIgnoreCase("false")) {
      } else {
        request.getVars().deletePageVar(value);
      }
      
      value = clearSessionVars.resolve(request).stringValue("");
      if (value.equals("*") || value.equalsIgnoreCase("true") || value.equalsIgnoreCase("all")) {
        request.getVars().removeAllSessionVars();
      } else if (value.equalsIgnoreCase("false")) {
      } else {
        request.getVars().deleteSessionVar(value);
      }
      
      if (params != null) {
        
        ConditionalListIterator cli = params.iterator(request);
        while (cli.hasMore()) {

          CommandParameter prm = (CommandParameter)cli.next();

          String val = request.getParameter(prm.name(),null);

          if (prm.notAllowed(request) && val != null) {
            throw new PageGenerationException("REQUEST_PARAMETER_NOT_ALLOWED",new String[]{prm.name()},null);
          }

          if (prm.mandatory(request) && val == null) {
            throw new PageGenerationException("MANDATORY_REQUEST_PARAMETER_MISSING",new String[]{prm.name()},null);
          }

          if (prm.nonBlank(request) && (val == null || val.length() == 0)) {
            throw new PageGenerationException("REQUEST_PARAMETER_IS_BLANK",new String[]{prm.name()},null);
          }

          if (prm.encrypted(request) && val != null && val.equals("DECRYPT_ERROR!")) {
            throw new PageGenerationException("REQUEST_PARAMETER_CANNOT_BE_DECRYPTED",new String[]{prm.name()},null);
          }

          if (val == null) val = prm.initialValue(request);

          switch (prm.scope(request)) {
            case 'p': request.getVars().setPageVar(prm.name(),val); break;
            case 's': request.getVars().setSessionVar(prm.name(),val); break;
          }

        }
      }
      
    } catch (Exception e) {
      throw new PageGenerationException(e);
    }
    
  }
  
  
}



