/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * CommandParameter.java
 *
 * Created on 12 July 2006 Wednesday, 22:03
 */

package org.moremotion.page;

import org.moremotion.config.ConditionalListItem;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigurationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.w3c.dom.Element;

/**
 * Represents a CommandParameter of a MoreMotion Page
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: CommandParameter.java 165 2008-07-01 15:29:42Z erkan $
 */
public class CommandParameter extends ConditionalListItem {
  
  private String name;
  private ConfigParameter mandatory = null;
  private ConfigParameter nonBlank = null;
  private ConfigParameter encrypted = null;
  private ConfigParameter notAllowed = null;
  private ConfigParameter scope = null;
  private ConfigParameter initialValue = null;
  
  
  protected String name() {
    return name;
  }
  
  protected boolean mandatory(MoreMotionRequest request) throws ConfigParameterException {
    return mandatory == null ? false : mandatory.resolve(request).booleanValue(false);
  }
  
  protected boolean nonBlank(MoreMotionRequest request) throws ConfigParameterException {
    return nonBlank == null ? false : nonBlank.resolve(request).booleanValue(false);
  }
  
  protected boolean encrypted(MoreMotionRequest request) throws ConfigParameterException {
    if (encrypted == null) return false;
    String key = encrypted.resolve(request).stringValue(null);
    return key == null || key.equals("false") ? false : true;
  }
  
  protected String encrkey(MoreMotionRequest request) throws ConfigParameterException {
    if (encrypted == null) return null;
    String key = encrypted.resolve(request).stringValue(null);
    if (key == null || key.equals("false")) return null;
    return key.equals("true") ? this.name : key;
  }
  
  protected boolean notAllowed(MoreMotionRequest request) throws ConfigParameterException {
    return notAllowed == null ? false : notAllowed.resolve(request).booleanValue(false);
  }
  
  protected char scope(MoreMotionRequest request) throws ConfigParameterException {
    return scope == null ? ' ' : scope.resolve(request).charValue(' ');
  }
  
  protected String initialValue(MoreMotionRequest request) throws ConfigParameterException {
    return initialValue == null ? null : initialValue.resolve(request).stringValue("");
  }
  
  protected CommandParameter(Element elm) throws ConfigurationException {
    
    super(elm);
    
    name = elm.getAttribute("name");
    
    String s = elm.getAttribute("scope");
    if (s.length() > 0) {
      scope = new ConfigParameter("scope",s);
    }
    
    s = elm.getAttribute("mandatory");
    if (s.length() > 0) {
      mandatory = new ConfigParameter("mandatory",s);
    }
    
    s = elm.getAttribute("nonBlank");
    if (s.length() > 0) {
      nonBlank = new ConfigParameter("nonBlank",s);
    }
    
    s = elm.getAttribute("encrypted");
    if (s.length() > 0) {
      encrypted = new ConfigParameter("encrypted",s);
    }
    
    s = elm.getAttribute("notAllowed");
    if (s.length() > 0) {
      notAllowed = new ConfigParameter("notAllowed",s);
    }
    
    s = elm.getAttribute("initialValue");
    initialValue = new ConfigParameter("initialValue",s);
    
  }
  
  public String toString() {
    StringBuffer buf = new StringBuffer();
    buf.append("CommandParameter [ name: " + this.name);
    buf.append(", mandatory: " + (mandatory == null ? "false" : mandatory.stringValue("false")) );
    buf.append(", nonBlank: " + (nonBlank == null ? "false" : nonBlank.stringValue("false")) );
    buf.append(", encrypted: " + (encrypted == null ? "false" : encrypted.stringValue("false")) );
    buf.append(", notAllowed: " + (notAllowed == null ? "false" : notAllowed.stringValue("false")) );
    buf.append(", scope: " + (scope == null ? " " : scope.stringValue(" ")) );
    buf.append(", initialValue: " + (initialValue == null ? "null" : initialValue.stringValue("")) );
    buf.append(" ]");
    return buf.toString();
  }
  
  
}

