/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * DataRequestNode.java
 *
 */

package org.moremotion.page;

import org.moremotion.config.ConditionalListItem;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.servlet.MoreMotionRequest;
import org.w3c.dom.Element;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: DataRequestNode.java 165 2008-07-01 15:29:42Z erkan $
 */
public class DataRequestNode extends ConditionalListItem {
  
  private String name;
  private ConfigParameter newName = null;
  private ConfigParameter encrypt = null;
  
  
  public String getName() {
    return name;
  }
  
  public String getNewName(MoreMotionRequest request) throws ConfigParameterException {
    return newName == null ? null : newName.resolve(request).stringValue(null);
  }
  
  public String getEncryptionKey(MoreMotionRequest request) throws ConfigParameterException {
    if (encrypt == null) return null;
    String key = encrypt.resolve(request).stringValue(null);
    if (key == null || key.equals("false")) return null;
    if (key.equals("true")) {
      key = getNewName(request);
      if (key == null) key = name;
    }
    return key;
  }
  
  public DataRequestNode(Element elm) {
    
    super(elm);
    
    name = elm.getAttribute("name");
    
    String s;
    
    s = elm.getAttribute("newName");
    if (s.length() > 0) {
      newName = new ConfigParameter("newName",s);
    }
    
    s = elm.getAttribute("encrypt");
    if (s.length() > 0) {
      encrypt = new ConfigParameter("encrypt",s);
    }
    
  }
  
  
  public String toString() {
    StringBuffer buf = new StringBuffer();
    buf.append("PageDatarequest [ name: " + this.name);
    buf.append(", encrypt: " + (encrypt == null ? "false" : encrypt.stringValue("false")) );
    buf.append(", newName: " + (newName == null ? "" : newName.stringValue("") ) );
    buf.append(" ]");
    return buf.toString();
  }
    
  
}
