/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * MScriptingProcess.java
 *
 * Created on February 10, 2003, 1:04 PM
 */

package org.moremotion.process;

import org.moremotion.evaluator.MMSymbolResolver;
import org.moremotion.evaluator.MScriptException;
import org.moremotion.evaluator.MScriptResolver;
import org.moremotion.evaluator.SymbolResolvingException;
import org.moremotion.util.XString;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: MScriptingProcess.java 171 2008-07-10 07:46:02Z mustafa $
 */

public class MScriptingProcess extends Process {
  
  private MMSymbolResolver mmsr;
  private boolean test = false;
  private StringBuffer sb;
  private MScriptResolver resolver;
  private String testPrm = "";
  
  
  public void prepare() throws ProcessException {
    
    try {
      
      mmsr = request.getSymbolResolver();
      
      if (logger.LEVEL1) {
        logger.printres("PROCESS_STARTED",callDef.getStepCaption());
      }
      
      String mscriptStr = callDef.getProcessConfig().getParameter("mscript").stringValue();
      
      if (mscriptStr.startsWith("_MSCRIPT_TEST ")) {
        
        test = true;
        testPrm = XString.after(mscriptStr,' ',null);
        if (testPrm != null) {
          mscriptStr = request.getParameter( testPrm, null );
          if (mscriptStr == null) {
            mscriptStr = mmsr.getValue("s:/mscripttest/" + testPrm);
          } else {
            mmsr.setValue("s:/mscripttest/" + testPrm, mscriptStr);
          }
          
        } else {
          mscriptStr = mmsr.getValue("s:/mscripttest/" + testPrm);
        }
      }
      
      if (logger.LEVEL2) {
        logger.printres("PROCESS_PARAMETERS",
            "\n MScript    : " + mscriptStr
            );
      }
      
      resolver = new MScriptResolver(mscriptStr,request.getSymbolResolver());
      
      sb = new StringBuffer(256);
      
    } catch (Exception ex) {
      
      if (logger.LEVEL1) {
        logger.printres("EXCEPTION_OCCURED", ex.getMessage());
      }
      createMessage(ex);
      setCompletionCode(12);
    }
    
  }
  
  public void processRecord(ProcessRecord prec) throws ProcessException {
    
    try {
      
      String s = resolver.resolve().replaceAll("\n","");
      
      if (sb.length() > 0) {
        sb.append("\n");
      }
      sb.append(s);
      
    } catch (MScriptException ex) {
      if (!test && logger.LEVEL1) {
        logger.printres("EXCEPTION_OCCURED", ex.getMessage());
      }
      throw new ProcessException(ex);
    }
    
  }
  
  
  public void finalizeProcess() throws ProcessException {
    
    if (logger.LEVEL2) logger.printres("PROCESS_FINALIZING",callDef.getStepCaption());
    
    if (test) {
      try {
        mmsr.setValue("s:/mscripttest/" + testPrm + "_RESULT", sb.toString());
      } catch (SymbolResolvingException ex) {
        createMessage(ex);
        setCompletionCode(12);
      }
    }
    
  }

  
}
