/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ProcessBlockPool.java
 *
 */

package org.moremotion.process;

import java.util.HashMap;
import org.moremotion.servlet.MoreMotionRequest;

/**
 * Keeps the process blocks created during one single request. A process block
 * can either be extracted from the process form coming with a request or can be
 * created by a query process.
 *
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ProcessBlockPool.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ProcessBlockPool {
  
  private HashMap blocks = null;
  private MoreMotionRequest request;
  
  private ProcessBlockPool() {
    blocks = new HashMap();
  }
  
  protected static MemoryProcessBlock getDummyBlock() {
    MemoryProcessBlock b = new MemoryProcessBlock(new String[]{});
    try {
      MemoryProcessRecord record = new MemoryProcessRecord();
      record.fields = new String[0];
      b.addRecord(record); 
    } catch (Exception e) {}
    return b;
  }
  
  protected static ProcessBlockPool getPool(MoreMotionRequest request) {
    
    ProcessBlockPool pool = (ProcessBlockPool)request.getAttribute("moremotion.process.block.pool");
    if (pool == null) {
      pool = new ProcessBlockPool();
      pool.request = request;
      request.setAttribute("moremotion.process.block.pool", pool);
    }
    
    return pool;
  }
  
  protected void addBlock(String blockName, ProcessBlock block) {
    
    this.blocks.put(blockName, block);
  } 
  
  protected ProcessBlock getBlock(String blockName) throws ProcessManagingException {
    
    ProcessBlock result = (ProcessBlock) blocks.get(blockName);
    if (result == null) {
      
      String pbinfoStr = request.getParameter("_pbinfo_" + blockName);
      if (pbinfoStr == null) {
        throw new ProcessManagingException("PROCESS_BLOCK_MISSING", new String[]{blockName},null);
      }
      result = new FormProcessBlock(request, blockName, pbinfoStr);
      blocks.put(blockName, result);
    }
    
    return result;
  }
  
}
