/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ProcessCommand.java
 */

package org.moremotion.process;

import java.util.HashMap;
import org.moremotion.config.Config;
import org.moremotion.config.ConfigNode;
import org.moremotion.config.ConfigNodeList;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigParameterException;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ProcessCommand.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ProcessCommand extends Config {
  
  private ConfigParameter nextPage;
  private ConfigParameter errorPage;
  private ConfigParameter continuePage;
  private ConfigParameter debugLevel; 
  private ConfigParameter templateName; 
  private ConfigParameter method; 
  
  public ProcessCommand(ConfigUnit unit) throws ConfigurationException {
    super(unit,false);
    parse(unit.getConfigNodeList());
  }
  
  private void parse(ConfigNodeList nodelist) throws ConfigParameterException {
    
    for (int i = 0; i < nodelist.size(); i++) {
      
      ConfigNode node = nodelist.get(i);
      
      String nodeName = node.getName();
      if (nodeName.equals("debugLevel")) {
        debugLevel = new ConfigParameter("debugLevel",node.getValue());
      }
      
      else if (nodeName.equals("nextPage")) {
        nextPage = new ConfigParameter("nextPage", node.getValue());
      }
      
      else if (nodeName.equals("errorPage")) {
        errorPage = new ConfigParameter("errorPage", node.getValue());
      }
      
      else if (nodeName.equals("continuePage")) {
        continuePage = new ConfigParameter("continuePage", node.getValue());
      }
      
      else if (nodeName.equals("method")) {
        method = new ConfigParameter("method", node.getValue());
      }
      
      else if (nodeName.equals("processTemplate")) {
        parseProcessTemplate((Element) node.getNode());
      }
      
      else {
        if (params == null) params = new HashMap();
        params.put(nodeName, new ConfigParameter(nodeName, node.getValue())) ;
      }
      
    }
    
    
  }
  
  private void parseProcessTemplate(Element elm) {
    
    templateName = new ConfigParameter("Process Template Name", elm.getAttribute("name"));
    NodeList nlist = elm.getElementsByTagName("param");
    
    if (nlist.getLength() > 0) {
      params = new HashMap(nlist.getLength());
      for (int i = 0; i < nlist.getLength(); i++) {
        Element node = (Element)nlist.item(i);
        params.put(node.getNodeName(), node.getNodeValue());
      }
    }
    
  }
    
  public String getTemplateName(MoreMotionRequest request) throws ConfigParameterException {
    if (templateName == null) {
      throw new ConfigParameterException("MISSING_CONFIGURATION_PARAMETER",new String[]{"process template name"},null);
    }
    return templateName.resolve(request).stringValue();
  }
  
  public String getNextPage() throws ConfigParameterException {
    return nextPage == null ? null : nextPage.stringValue();
  }
  
  public String getErrorPage() throws ConfigParameterException {
    return errorPage == null ? null : errorPage.stringValue(null);
  }
  
  public String getContinuePage() throws ConfigParameterException {
    return continuePage == null ? null : continuePage.stringValue(null);
  }
  
  public int getDebugLevel(MoreMotionRequest request) throws ConfigParameterException {
    return debugLevel == null ? 0 : debugLevel.resolve(request).intValue(0);
  }
  
  public String getMethod(MoreMotionRequest request) throws ConfigParameterException {
    return method == null ? null : method.resolve(request).stringValue(null);
  }
  
  public HashMap getParams() {
    return params;
  }  
  
  
}
