/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ProcessCommandManager.java
 *
 * Created on 11 April 2007 Wednesday, 12:09
 *
 */

package org.moremotion.process;

import org.moremotion.config.ConfigCache;
import org.moremotion.config.ConfigNodeList;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ProcessCommandManager.java 115 2008-04-08 11:56:01Z erkan $
 */
public class ProcessCommandManager {
  
  public ProcessCommandManager() {
  }
  
  public static ProcessCommand getProcessCommand(String name) throws ConfigurationException {
    return getProcessCommand("processCommand", name);
  }
    
  public static ProcessCommand getAjaxProcessCommand(String name) throws ConfigurationException {
    return getProcessCommand("ajaxProcessCommand", name);
  }
    
  public static ProcessCommand getProcessCommand(String type, String name) throws ConfigurationException {
    
    ConfigUnit unit = ConfigCache.getUnit(type, name);
    if (unit == null) {
      return null;
    }
    
    if (unit.isNew()) {
      synchronized (unit.getSyncObject()) {
        parse(unit);
      }
    }
    
    return (ProcessCommand)unit.getDataObject();
  }
  
  
  
  private static void parse(ConfigUnit unit) throws ConfigurationException {
    
    ConfigNodeList nodelist = unit.getConfigNodeList();
    
    ProcessCommand command = new ProcessCommand(unit);
    unit.setDataObject(command);
    
  }
  
  
}
