/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * ProcessTemplateCallDefinition.java
 *
 * Created on 12 March 2008 Wednesday, 16:55
 */

package org.moremotion.process;

import java.util.ArrayList;
import java.util.HashMap;
import org.moremotion.config.ConfigCache;
import org.moremotion.config.ConfigParameter;
import org.moremotion.config.ConfigUnit;
import org.moremotion.config.ConfigurationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.evaluator.MScriptResolver;
import org.moremotion.util.XString;
import org.w3c.dom.Element;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: ProcessTemplateCallDefinition.java 51 2008-03-10 11:46:54Z erkan $
 */
public class ProcessTemplateCallDefinition extends CallDefinition {
  
  private ProcessTemplateConfig templateConfig = null;
  private boolean targetIsDynamic = false;
  private HashMap templateCallParams = null;
  
  public ProcessTemplateCallDefinition(Element elm, Object parent) throws ConfigurationException {
    super(elm, parent);
    if (elm != null) {
      targetIsDynamic = MScriptResolver.hasFunctions(this.getTargetName());
      prepareTemplateCallParams();
    }
  }
  
  protected boolean isTargetDynamic() {
    return this.targetIsDynamic;
  }
  
  protected void setTemplateConfig(ProcessTemplateConfig config) {
    this.templateConfig = config;
    if (config != null) this.setTargetName(config.getName());
  }
  
  protected HashMap getTemplateCallParams() {
    return this.templateCallParams;
  }
  
  protected ProcessTemplateConfig getTemplateConfig() {
    return this.templateConfig;  
  }
  
  protected ProcessTemplateConfig getTemplateConfig(MoreMotionRequest request) throws ConfigurationException {
    if (!targetIsDynamic) {
      if (this.templateConfig == null || this.templateConfig.getConfigUnit().getDataObject() == null) {
        this.templateConfig = _getTemplateConfig(this.getTargetName(), getPageName());
      }
      return this.templateConfig;
    } else {
      try {
        String _targetName = request.resolve(this.getTargetName());
        if (_targetName.trim().length() == 0) {
          throw new ConfigurationException("TEMPLATE_NAME_MISSING",new String[]{this.getTargetName()},null);
        }
        return _getTemplateConfig(_targetName, getPageName());
      } catch (Exception ex) {
        throw new ConfigurationException("DYNAMIC_TEMPLATE_NAME_ERROR",new String[]{this.getTargetName()},ex);
      }
    }
  }
  
  private ProcessTemplateConfig _getTemplateConfig(String name, String page) throws ConfigurationException {
    
    ConfigUnit unit = null;
    if (name.charAt(0) != '/' && page != null) {
      unit = ConfigCache.getUnit("processTemplate", page + name);
    }
    
    if (unit == null) {
      unit = ConfigCache.getUnit("processTemplate",XString.after(name,'/'));
    }
    
    if (unit == null) {
      throw new ConfigurationException("PROCESS_TEMPLATE_MISSING",new String[]{name},null);
    }
    
    ProcessTemplateConfig config = (ProcessTemplateConfig)unit.getDataObject();
    if (config == null) {
      config = new ProcessTemplateConfig(unit);  
    }
    
    return config;
        
  }
  
  
  protected void prepareTemplateCallParams() {
    
    ArrayList list = this.getParams();
    if (list != null && list.size() > 0) {
      this.templateCallParams = new HashMap();
      for (int i = 0; i < list.size(); i++) {
        Element elm = (Element) list.get(i);
        String n = elm.getAttribute("name");
        String v = elm.getFirstChild() == null ? "" : elm.getFirstChild().getNodeValue();
        if (MScriptResolver.hasFunctions(v)) {
          ConfigParameter p = new ConfigParameter(n,v);
          this.templateCallParams.put(n, p);
        } else {
          this.templateCallParams.put(n, v);
        }
      }
    }
    
  }

  
}
