/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

package org.moremotion.security;

import java.util.*;


/** 
 * CheckPoint describes an authority to perform a task. 
 * @author Yusuf Boyacigil
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: CheckPoint.java 51 2008-03-10 11:46:54Z erkan $
 */
public class CheckPoint implements Comparable {
  private String id = null;
  private String name;
  private String description;
  private Properties attributes = null;
  
  /** Creates a new check point.
   */  
  public CheckPoint() {}  
  
  /** Creates a new check point.
   * @param name Check point name.
   * @param description Check point description.
   */  
  public CheckPoint(String name, String description) {  
    this.name = name;
    this.description = description;
  }
  
  /** Returns check point name.
   */  
  public String getName() {
    return name;
  }
  
  /** Returns check point description.
   */  
  public String getDescription() {
    return description;
  }
  
  /** Sets check point name.
   */  
  public void setName(String name) {
    this.name = name;
  }
  
  /** Sets check point description.
   */  
  public void setDescription(String description) {
    this.description = description;
  }
  
  /** Returns check point identifier.
   */  
  public String getId() {  
    return this.id;
  }

  /** Sets check point identifier.
   * <P><STRONG>For internal use only.</STRONG></P>
   */  
  public void setId(String value) {
    this.id = value;
  }
  
  /** 
   * Compares the name of this checkpoint with the name of the given one and returns <CODE>true</CODE> when they are equal.
   */  
  public boolean equals(Object obj) {
    if (obj instanceof CheckPoint) {
      CheckPoint cp = (CheckPoint) obj;
      return name.equals(cp.getName());
    }
    return false;
  }  
  
  /** 
   * Returns the string respresentation of this checkpoint.
   */  
  public String toString() {
    String result = "CheckPoint {" + 
                    "\n id: " + getId() + 
                    "\n name: " + getName() + 
                    "\n description: " + getDescription() +
                    "\n}";
    return result;
  }
  
  /** Compares this check point to other.
   */  
  public int compareTo(java.lang.Object obj) {
    int result = 0;
    if (obj instanceof CheckPoint) {
      result = name.compareTo(((CheckPoint) obj).getName());
    }
    return result;
  }
  
  
  /**
   * Returns the attribute of this checkpoint.
   */
  public Properties getAttributes() {
    return this.attributes;
  }
  
  /**
   * Sets a user attribute to this checkpoint.
   */
  public void setAttribute(String name, String value) {
    if (attributes == null) {
      attributes = new Properties();
    }
    attributes.setProperty(name, value);
  }
  
  /**
   * Returns the value of the specified checkpoint attribute.
   */
  public String getAttribute(String name) {
    if (attributes == null) return null;
    return attributes.getProperty(name);
  }
  
  /**
   * Removes the specified attribute from this checkpoint.
   */
  public void removeAttribute(String name) {
    if (attributes != null) {
      attributes.remove(name);
    }
  }
  
}