/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * RememberUser.java
 *
 * Created on 25 January 2008 Friday, 23:45
 *
 */

package org.moremotion.security;

import org.moremotion.evaluator.EvaluationException;
import org.moremotion.servlet.MoreMotionRequest;
import org.moremotion.util.XString;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: RememberUser.java 51 2008-03-10 11:46:54Z erkan $
 */
public class RememberUser {
  
  public static final String REMEMBER_OPTION_COOKIE_PREFIX = "moremotion.remember.user.option.";
  public static final String RECENT_USERNAME_COOKIE_PREFIX = "moremotion.recent.username.";
  public static final String RECENT_SUBDOMAIN_COOKIE_PREFIX = "moremotion.recent.subdomain.";
  public static final String RECENT_LOGINSTATUS_COOKIE_PREFIX = "moremotion.recent.loginstatus.";
  
  private MoreMotionRequest request = null;
  private String domain = "main";
  private String subdomain = null;
  private int configOption = 1;
  private int usersChoice = -1;
  private int cookieAge = -2;
  private String userName = null;
  private boolean loginCalledBefore = false;
  
  public RememberUser(MoreMotionRequest request, String domain) {
    this.request = request;
    this.domain = domain;
  }
  
  public void setOption(String option, int cookieAge) {
    this.cookieAge = cookieAge;
    this.configOption = parseConfigOption(option);
  }
  
  private void setUsersChoice() {
    if (this.configOption == 4) {
      try {
        this.usersChoice = parseUsersChoice(request.getSymbolResolver().getValue("rememberme"));
        request.setCookieValue(REMEMBER_OPTION_COOKIE_PREFIX + domain, getOption() +"", cookieAge);
      } catch (EvaluationException ex) {}
    }
  }
  
  public void setUser(String userName, String subdomain) {
    this.userName = userName;
    this.subdomain = subdomain;
  }
  
  public void login() {
    setUsersChoice();
    request.setCookieValue(RECENT_LOGINSTATUS_COOKIE_PREFIX + domain, "true", cookieAge);
    flush();
  }
  
  public void logout() {
    request.setCookieValue(RECENT_LOGINSTATUS_COOKIE_PREFIX + domain, "", 0);
    flush();
  }
  
  private int parseConfigOption(String option) {
    String s = XString.before(option,".");
    if (s.equals("2")) return 2;
    if (s.equals("3")) return 3;
    if (s.equals("4")) return 4;
    return 1;
  }
  
  private int parseUsersChoice(String option) {
    if (option == null) return 1;
    String s = XString.before(option,".");
    if (s.equals("1")) return 1;
    if (s.equals("2")) return 2;
    if (s.equals("3")) return 3;
    if (s.equals("false")) return 1;
    if (s.equals("name")) return 2;
    if (s.equals("true")) return 3;
    return 1;
  }
  
  public String getRecentUserName() {
    if (getOption() == 1) return null;
    if (userName == null) {
      userName = request.getCookieValue(RECENT_USERNAME_COOKIE_PREFIX + domain);
      if (userName != null && userName.length() == 0) return null;
    }
    return userName;
  }
  
  public String getRecentSubDomain() {
    if (getOption() == 1) return null;
    if (subdomain == null) {
      subdomain = request.getCookieValue(RECENT_SUBDOMAIN_COOKIE_PREFIX + domain);
      if (subdomain == null) return "";
    }
    return subdomain;
  }
  
  public int getOption() {
    if (configOption == 4) {
      if (usersChoice == -1) {
        usersChoice = parseUsersChoice(request.getCookieValue(REMEMBER_OPTION_COOKIE_PREFIX + domain));
      }
      return usersChoice;
    }
    return configOption;
  }
  
  public boolean isLoggedIn() {
    return getOption() == 3 && request.getCookieValue(RECENT_LOGINSTATUS_COOKIE_PREFIX + domain).equals("true");
  }
  
  public void touchCookie() {
    flush();
  }
  
  public void flush() {
    
    if (getOption() == 1) {
      clearCookie();
      
    } else {
      setCookie();
      
    }
    
  }
  
  private void clearCookie() {
    // request.setCookieValue(REMEMBER_OPTION_COOKIE_PREFIX + domain, "", 0);
    request.setCookieValue(RECENT_USERNAME_COOKIE_PREFIX + domain, "", 0);
    request.setCookieValue(RECENT_SUBDOMAIN_COOKIE_PREFIX + domain, "", 0);
  }
  
  private void setCookie() {
    // request.setCookieValue(REMEMBER_OPTION_COOKIE_PREFIX + domain, getOption() +"", cookieAge);
    request.setCookieValue(RECENT_USERNAME_COOKIE_PREFIX + domain, userName, cookieAge);
    request.setCookieValue(RECENT_SUBDOMAIN_COOKIE_PREFIX + domain, subdomain, cookieAge);
  }
  
  
}
