/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * Logger.java
 *
 * Created on 29 June 2004 Tuesday, 00:30
 */

package org.moremotion.util;

import java.util.Date;
import java.util.StringTokenizer;
import org.moremotion.resource.Resource;
import org.moremotion.resource.ResourceException;
import org.moremotion.servlet.SystemConfig;
import java.text.SimpleDateFormat;

/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: Logger.java 51 2008-03-10 11:46:54Z erkan $
 */
public class Logger {

  private String debugId = null;
  private int idLength = 25;
  private static final String SPACE = "                                                              ";
  private int debugLevel = 0;
  private int indent = 0;
  public boolean LEVEL1 = false;
  public boolean LEVEL2 = false;
  public boolean LEVEL3 = false;
  
  private String resourceBundleName = null;


  /**
   * Creates a new instance of Logger
   */
  public Logger(Object debugId) {

    String s = null;
    if (debugId instanceof String) {
      s = (String)debugId;
    } else {
      s = debugId.getClass().getName();
    }
    setDebugId(s);
  }

  public void setLevel(int debugLevel) {
    if (SystemConfig.disableDebug()) return;
    this.debugLevel = debugLevel;
    this.LEVEL1 = debugLevel > 0;
    this.LEVEL2 = debugLevel > 1;
    this.LEVEL3 = debugLevel > 2;
  }

  public int getDebugLevel() {
    return this.debugLevel;
  }

  public Logger(Object debugId, int debugLevel) {
    this(debugId);
    setLevel(debugLevel);
  }

  public Logger(String debugId) {
    setDebugId(debugId);
  }
  
  private void setDebugId(String id) {
    if (id.length() > idLength) {
      int spos = id.length() - idLength + 1;
      this.debugId = "?" + id.substring(spos);
    } else {
      String s = id + SPACE;
      this.debugId = s.substring(0,idLength);
    }
  }

  public void setResourceBundleName(String bundleName) {
    this.resourceBundleName = bundleName;
  }
  
  public String getResourceBundleName() {
    return this.resourceBundleName;
  }
  
  public String getId() {
    return this.debugId;
  }

  public void incrIndent() {
    if (debugLevel > 0) {
      indent++;
    }
  }

  public void decrIndent() {
    if (debugLevel > 0) {
      indent--;
    }
    if (indent < 0) indent = 0;
  }

  public void setIndent(int indent) {
    this.indent = indent;
  }

  public int getIndent() {
    return this.indent;
  }

  public int getInnerIndent() {
    return this.debugLevel == 0 ? this.indent : this.indent + 1;
  }

  private String getIndentSpace() {
    if (indent == 0) return "";
    return "                                                               ".substring(0,(indent*2));
  }

  public void setIdLength(int value) {
    idLength = value;
    debugId = debugId + SPACE;
    debugId = debugId.substring(0,value);
  }

  public void print(String s) {
    
    SimpleDateFormat format = new SimpleDateFormat("MM/dd HH:mm:ss");
    String ds = format.format(new Date());
    
    StringTokenizer t = new StringTokenizer(s,"\n");
    System.out.println(ds + " | " + this.debugId + " | " + getIndentSpace() + t.nextToken());
    while (t.hasMoreTokens()) {
      System.out.println(SPACE.substring(0,idLength) + "                  | " + getIndentSpace() + t.nextToken());
    }

  }

  public void print(String s, String seperator) {
    System.out.println(SPACE.substring(0,idLength) + "                  | " + seperator);
    print(getIndentSpace() + s);
  }
  
  /**
   * Formats the given resource with the given arguments and prints it.
   */
  public void printres(String resourceId, String[] arguments) {
    
    String resbund = XString.before(resourceId,"*",null);
    
    if (resbund != null) {
      if (resbund.length() == 0) resbund = "moremotion";
      resourceId = XString.after(resourceId,"*");
    } else {
      resbund = resourceBundleName;
    }
    
    printres(resbund, resourceId, arguments);
  }

  public void printres(String resourceId) {
    printres(resourceId, new String[]{});
  }
  
  public void printres(String resourceId, String arg1) {
    printres(resourceId, new String[]{arg1});
  }
  
  public void printres(String resourceId, String arg1, String arg2) {
    printres(resourceId, new String[]{arg1, arg2});
  }
  
  public void printres(String resourceId, String arg1, String arg2, String arg3) {
    printres(resourceId, new String[]{arg1, arg2, arg3});
  }
  
  private void printres(String resourceBundleName, String resourceId, String[] arguments) {
    Resource res = new Resource(resourceId,arguments,resourceBundleName);
    try {
      print(res.format(SystemConfig.applicationLanguage()));
    } catch (ResourceException ex) {
      ex.printStackTrace();
    }
  }

  public static void main(String[] args) throws Exception {
  }


}
