/*
 *********************************************************************************
 *                                                                               *
 *  MoreMotion Application Framework                                             *
 *                                                                               * 
 *  Copyright (C) 2002-2008 MOR YAZILIM, (http://www.moryazilim.com)             *
 *                                                                               *
 *  This file is a part of MoreMotion Application Framework                      *
 *                                                                               *
 *  MoreMotion Application Framework is free software: you can redistribute      * 
 *  it and/or modify it under the terms of the GNU Lesser Genaral Public License * 
 *  as published by the Free Software Foundation, either version 3 of the        *
 *  License, or (at your option) any later version.                              *
 *                                                                               *
 *  MoreMotion Application Framework is distributed in the hope that it will     *
 *  be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of    *
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the                 *
 *  GNU General Public License for more details.                                 *
 *                                                                               *
 *  You should have received a copy of the GNU Lesser General Public License     *
 *  along with MoreMotion Application Framework.                                 * 
 *  If not, see http://www.gnu.org/licenses/lgpl.txt                             *
 *                                                                               *
 *********************************************************************************
 */

/*
 * SignedUser.java
 *
 * Created on 13 February 2007 Tuesday, 20:44
 */

package org.moremotion.security.domain;

import java.util.Date;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;


/**
 * @author Erkan Cengiz (erkan.cengiz@moryazilim.com)
 * @version $Id: SignedUser.java 51 2008-03-10 11:46:54Z erkan $
 */
public class SignedUser {
  
  public Hashtable ownerUserList;
  public Hashtable sessions;
  public String userName;
  public long lmodified;
  
  public SignedUser(String userName, String sid, Hashtable ownerUserList) {
    this.ownerUserList = ownerUserList;
    this.userName = userName;
    sessions = new Hashtable();
    lmodified = (new Date()).getTime();
  }
  
  protected void touch(String sid) {
    
    SessionInfo si = (SessionInfo)sessions.get(sid);
    if (si == null) {
      si = new SessionInfo();
      si.signInTime = (new Date()).getTime();
      sessions.put(sid,si);
    }
    si.lastReferredTime = (new Date()).getTime();
  }
  
  public long getSignInTime() {
    long value = 99999999999999L;
    Iterator itr = sessions.entrySet().iterator();
    while ( itr.hasNext() ) {
      Map.Entry entry = (Map.Entry)itr.next();
      SessionInfo si = (SessionInfo)entry.getValue();
      if (si.signInTime < value) {
        value = si.signInTime;
      }
    }
    return value;
  }
  
  public long getLastReferredTime() {
    long value = 0;
    Iterator itr = sessions.entrySet().iterator();
    while ( itr.hasNext() ) {
      Map.Entry entry = (Map.Entry)itr.next();
      SessionInfo si = (SessionInfo)entry.getValue();
      if (si.lastReferredTime > value) {
        value = si.lastReferredTime;
      }
    }
    return value;
  }
  
  public void killOthers(String Sid, Hashtable globalSessions, Hashtable killedSessions) {
    Iterator itr = sessions.entrySet().iterator();
    while ( itr.hasNext() ) {
      Map.Entry entry = (Map.Entry)itr.next();
      String sid = (String)entry.getKey();
      if (Sid == null || !Sid.equals(sid)) {
        globalSessions.remove(sid);
        killedSessions.put(sid,"");
        itr.remove();
      }
    }
  }
  
  public void signOut(String sid) {
    SessionInfo si = (SessionInfo)sessions.get(sid);
    if (si == null) {
      return;
    }
    sessions.remove(sid);
  }
  
  public void signIn(String sid) {
    SessionInfo si = (SessionInfo)sessions.get(sid);
    if (si == null) {
      si = new SessionInfo();
      si.signInTime = (new Date()).getTime();
      si.lastReferredTime = (new Date()).getTime();
      sessions.put(sid,si);
    } else {
      si.signInTime = (new Date()).getTime();
      si.lastReferredTime = (new Date()).getTime();
    }
  }
  
  public SessionInfo getDomainEntryInfo(String sid) {
    return (SessionInfo)sessions.get(sid);  
  }
  
  public int getSessionCount() {
    if (sessions == null)  {
      return 0;
    }
    return sessions.size();
  }
  
  public String toString() {
    StringBuffer sb = new StringBuffer();
    sb.append("User Name : " + userName + "\n");
    sb.append("Sessions : \n");
    Iterator itr = sessions.entrySet().iterator();
    while ( itr.hasNext() ) {
      Map.Entry entry = (Map.Entry)itr.next();
      String sid = (String)entry.getKey();
      SessionInfo si = (SessionInfo)entry.getValue();
      sb.append("  " + sid + ": {" + si + "} \n");
    }
    return sb.toString();
  }
  
  
}
